/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.communication.jndi.client;

import java.util.Hashtable;
import java.util.logging.Logger;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NameAlreadyBoundException;
import javax.naming.NameNotFoundException;
import javax.naming.NamingException;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.configuration.DomainConfiguration;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.communication.jndi.agent.JNDIAgentService;

/**
 * Fractal component used to retrieve the JNDI initial context.
 * 
 * @author Adrien Louis - EBM WebSourcing
 * @author Christophe HAMERLING - EBM WebSourcing
 * @author Roland NAUDIN - EBM WebSourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = JNDIService.class))
public class JNDIServiceImpl implements JNDIService {

    /**
     * Users context
     */
    protected Context usersContext;

    /**
     * Logging utilities.
     */
    protected LoggingUtil log;

    /**
     * The JNDI Agent service fractal component
     */
    @Requires(name = "jndiAgent")
    private JNDIAgentService jndiAgentService;

    /**
     * The Configuration service fractal component
     */
    @Requires(name = "configuration")
    private ConfigurationService configurationService;

    /**
     * The domain configuration
     */
    private DomainConfiguration domainConfiguration;

    private InitialContextDecorator initialContextDecorator;

    @LifeCycle(on = LifeCycleType.START)
    protected void start() {
        this.log = new LoggingUtil(Logger.getLogger(Constants.FRACTAL_COMPONENT_LOGGER_NAME));
        this.log.start();

        this.domainConfiguration = this.configurationService.getDomainConfiguration();

        this.log.end();
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() {
        this.log.start();

        if (this.initialContextDecorator != null) {
            try {
                this.initialContextDecorator.closeInitialContextDecorator();
            } catch (final NamingException e) {
                this.log.error(e);
            } finally {
                this.initialContextDecorator = null;
            }
        }

        this.log.end();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.kernel.jndi.JNDIService#getJNDIContext()
     */
    public synchronized InitialContext getInitialContext() throws NamingException {
        //lazy synchronized on this method        
        this.log.call();

        if (this.initialContextDecorator == null) {
            this.log.debug("Create a new InitialContext decorator");
            final Hashtable<String, Object> env = this.getJNDIEnv();
            this.log.debug("JNDI properties used : " + env);

            this.initialContextDecorator = new InitialContextDecorator(env, this.log, this.jndiAgentService);

        }

        return this.initialContextDecorator;
    }

    /*
     * (non-Javadoc)
     * @see org.ow2.petals.communication.jndi.client.JNDIService#getUsersContext()
     */
    public synchronized Context getUsersContext() throws NamingException {
      //lazy synchronized on this method
        this.log.call();
        
        if (this.usersContext == null) {
            try {
                this.usersContext = (Context) this.getInitialContext().lookup(USERS_REF);
            } catch (final NameNotFoundException e) {
                this.usersContext = this.getInitialContext().createSubcontext(USERS_REF);
            } catch (final NameAlreadyBoundException e) {
                // in distributed mode, it is possible that 2 containers are
                // creating the context in the same time, the second
                // container get an Already Bound exception
                this.usersContext = (Context) this.getInitialContext().lookup(USERS_REF);
            }
        }
        return this.usersContext;
    }

    /**
     * Construct and return the JNDI environment values. If no JNDI server is
     * specified in the Domain topology, use the internal JNDI server
     * 
     * @return The environment hashTable
     */
    private Hashtable<String, Object> getJNDIEnv() {
        final Hashtable<String, Object> env = new Hashtable<String, Object>();

        if ((this.domainConfiguration.getJndiConfiguration() == null) ||
                (this.domainConfiguration.getJndiConfiguration().getJndiFactory() == null)) {
            env.put(JNDIAgentService.ENVIRONMENT_LOCAL_CONNECTION, Boolean.toString(true));
            env.put(Context.INITIAL_CONTEXT_FACTORY, JNDIAgentService.JNDI_PETALS_FACTORY);
        } else {
            env.put(Context.INITIAL_CONTEXT_FACTORY, this.domainConfiguration.getJndiConfiguration().getJndiFactory());
            env.put(Context.PROVIDER_URL, this.domainConfiguration.getJndiConfiguration().getJndiProviderUrl().toString());
            env.put(Context.SECURITY_PRINCIPAL, this.domainConfiguration.getJndiConfiguration().getJndiSecurityPrincipal());
            env.put(Context.SECURITY_CREDENTIALS, this.domainConfiguration.getJndiConfiguration().getJndiSecurityCredentials());
            env.put(Context.BATCHSIZE, String.valueOf(this.domainConfiguration.getJndiConfiguration().getJndiBatchSize()));
            env.put("java.naming.poolsize", String.valueOf(this.domainConfiguration.getJndiConfiguration().getJndiPoolSize()));
            env.put(JAVA_NAMING_FACTORY_LOGGER, this.log);
        }
        return env;
    }

}
