/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.component.context;

import java.util.MissingResourceException;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.management.MBeanNames;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.MessagingException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.MBeanServer;
import javax.naming.InitialContext;
import javax.xml.namespace.QName;

import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;

/**
 * Decorator class of <code>ComponentContext</code> for the use of
 * <code>InstallationContext</code>. Only getMBeanNames, getMBeanServer,
 * getNamingContext, getTransactionContext are allowed. Other methods throw a
 * <code>NoSuchMethodError</code> exception
 * 
 * @author Adrien LOUIS - EBM WebSourcing
 */
public class ComponentContextForInstallation implements ComponentContext {
    protected ComponentContext context;

    /**
     * Constructor
     * 
     * @param context
     *      The real component context to decorate
     */
    public ComponentContextForInstallation(ComponentContext context) {
        this.context = context;
    }

    /**
     * @see javax.jbi.component.ComponentContext#activateEndpoint(javax.xml.
     *  namespace.QName, java.lang.String)
     */
    public ServiceEndpoint activateEndpoint(QName serviceName, String endpointName)
            throws JBIException {
        throw new IllegalStateException("Not allowed in InstallationContext");
    }

    /**
     * @see javax.jbi.component.ComponentContext#deactivateEndpoint(javax.jbi.
     *  servicedesc.ServiceEndpoint)
     */
    public void deactivateEndpoint(ServiceEndpoint endpoint) throws JBIException {
        throw new IllegalStateException("Not allowed in InstallationContext");
    }

    /**
     * @see
     *  javax.jbi.component.ComponentContext#deregisterExternalEndpoint(javax
     *  .jbi.servicedesc.ServiceEndpoint)
     */
    public void deregisterExternalEndpoint(ServiceEndpoint externalEndpoint) throws JBIException {
        throw new IllegalStateException("Not allowed in InstallationContext");
    }

    /**
     * @see javax.jbi.component.ComponentContext#getComponentName()
     */
    public String getComponentName() {
        throw new IllegalStateException("Not allowed in InstallationContext");
    }

    /**
     * @see javax.jbi.component.ComponentContext#getDeliveryChannel()
     */
    public DeliveryChannel getDeliveryChannel() throws MessagingException {
        throw new IllegalStateException("Not allowed in InstallationContext");
    }

    /**
     * @see
     *  javax.jbi.component.ComponentContext#getEndpoint(javax.xml.namespace.
     *  QName, java.lang.String)
     */
    public ServiceEndpoint getEndpoint(QName service, String name) {
        throw new IllegalStateException("Not allowed in InstallationContext");
    }

    /**
     * @see
     *  javax.jbi.component.ComponentContext#getEndpointDescriptor(javax.jbi.
     *  servicedesc.ServiceEndpoint)
     */
    public Document getEndpointDescriptor(ServiceEndpoint endpoint) throws JBIException {
        throw new IllegalStateException("Not allowed in InstallationContext");
    }

    /**
     * @see
     *  javax.jbi.component.ComponentContext#getEndpoints(javax.xml.namespace
     *  .QName)
     */
    public ServiceEndpoint[] getEndpoints(QName interfaceName) {
        throw new IllegalStateException("Not allowed in InstallationContext");
    }

    /**
     * @see
     *  javax.jbi.component.ComponentContext#getEndpointsForService(javax.xml
     *  .namespace.QName)
     */
    public ServiceEndpoint[] getEndpointsForService(QName serviceName) {
        throw new IllegalStateException("Not allowed in InstallationContext");
    }

    /**
     * @see javax.jbi.component.ComponentContext#getExternalEndpoints(javax.xml.
     *  namespace.QName)
     */
    public ServiceEndpoint[] getExternalEndpoints(QName interfaceName) {
        throw new IllegalStateException("Not allowed in InstallationContext");
    }

    /**
     * @see javax.jbi.component.ComponentContext#getExternalEndpointsForService(
     *  javax.xml.namespace.QName)
     */
    public ServiceEndpoint[] getExternalEndpointsForService(QName serviceName) {
        throw new IllegalStateException("Not allowed in InstallationContext");
    }

    /**
     * TODO : not allowed in the specification but necessary to run SUN example
     * 
     * @see javax.jbi.component.ComponentContext#getInstallRoot()
     */
    public String getInstallRoot() {
        return this.context.getInstallRoot();
        // throw new IllegalStateException("Not allowed in
        // InstallationContext");
    }

    /**
     * TODO : not allowed in the specification but necessary to manage the
     * loggers properly
     * 
     * @see javax.jbi.component.ComponentContext#getLogger(java.lang.String,
     *  java.lang.String)
     */
    public Logger getLogger(String suffix, String resourceBundleName)
            throws MissingResourceException, JBIException {
        // throw new JBIException("Not allowed in InstallationContext");
        return this.context.getLogger(suffix, resourceBundleName);
    }

    /**
     * @see javax.jbi.component.ComponentContext#getMBeanNames()
     */
    public MBeanNames getMBeanNames() {
        return this.context.getMBeanNames();
    }

    /**
     * @see javax.jbi.component.ComponentContext#getMBeanServer()
     */
    public MBeanServer getMBeanServer() {
        return this.context.getMBeanServer();
    }

    /**
     * @see javax.jbi.component.ComponentContext#getNamingContext()
     */
    public InitialContext getNamingContext() {
        return this.context.getNamingContext();
    }

    /**
     * @see javax.jbi.component.ComponentContext#getTransactionManager()
     */
    public Object getTransactionManager() {
        return this.context.getTransactionManager();
    }

    /**
     * @see javax.jbi.component.ComponentContext#getWorkspaceRoot()
     */
    public String getWorkspaceRoot() {
        return this.context.getWorkspaceRoot();
    }

    /**
     * @see
     *  javax.jbi.component.ComponentContext#registerExternalEndpoint(javax.jbi
     *  .servicedesc.ServiceEndpoint)
     */
    public void registerExternalEndpoint(ServiceEndpoint externalEndpoint) {
        throw new IllegalStateException("Not allowed in InstallationContext");
    }

    /**
     * @see
     *  javax.jbi.component.ComponentContext#resolveEndpointReference(org.w3c
     *  .dom.DocumentFragment, javax.jbi.component.Capabilities)
     */
    public ServiceEndpoint resolveEndpointReference(DocumentFragment epr) {
        throw new IllegalStateException("Not allowed in InstallationContext");
    }

}
