/**
 * Copyright (c) 2005-2012 Fossil E-Commerce, 2005-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.component.context;

import static org.ow2.petals.microkernel.api.Constants.UNRESOLVED_INTERFACE;
import static org.ow2.petals.microkernel.container.ContainerService.COMPONENT_LOGGER_PREFIX;

import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.Set;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.jbi.management.MBeanNames;
import javax.jbi.messaging.MessagingException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.MBeanServer;
import javax.naming.InitialContext;
import javax.xml.namespace.QName;

import org.ow2.easywsdl.extensions.wsdl4complexwsdl.WSDL4ComplexWsdlReaders;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlException;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlReader;
import org.ow2.easywsdl.wsdl.api.Description;
import org.ow2.easywsdl.wsdl.api.Service;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.jbi.descriptor.original.generated.LinkType;
import org.ow2.petals.microkernel.api.container.ComponentContextCommunication;
import org.ow2.petals.microkernel.api.container.Installer;
import org.ow2.petals.microkernel.api.jbi.ComponentContext;
import org.ow2.petals.microkernel.api.jbi.messaging.DeliveryChannel;
import org.ow2.petals.microkernel.api.jbi.messaging.RouterService;
import org.ow2.petals.microkernel.api.jbi.messaging.registry.RegistryException;
import org.ow2.petals.microkernel.api.jbi.security.SecurityContext;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.jbi.messaging.exchange.DeliveryChannelImpl;
import org.ow2.petals.microkernel.util.ParameterCheckHelper;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;

/**
 * @author Rafael Marins - Fossil E-Commerce
 * @author wjoseph - EBM WebSourcing
 */
public class ComponentContextImpl implements ComponentContext {

    protected DeliveryChannel deliveryChannel;

    /**
     * references of registered external serviceEndpoints
     */
    protected Set<ServiceEndpoint> externalEndpoints;

    /**
     * references of registered internal serviceEndpoints
     */
    protected Set<ServiceEndpoint> internalEndpoints;

    protected Component jbiComponent;

    /**
     * The component context communication (can be either an installer or a
     * component life cycle)
     */
    protected final ComponentContextCommunication componentContextCommunication;

    protected LoggingUtil logger;

    /**
     * Component loggers
     */
    private final Map<String, String> componentLoggerRBNames;

    private SecurityContext securityContext;

    /**
     * Constructor of the ComponentContextImpl class
     * 
     * @param fractalComponent
     *            the fractal representation of installer
     * @throws IllegalArgumentException
     *             Fractal Error: Impossible to get the service interface
     */
    public ComponentContextImpl(ComponentContextCommunication componentContextCommunication) {
        super();
        this.componentContextCommunication = componentContextCommunication;
        this.logger = new LoggingUtil(
                java.util.logging.Logger
                        .getLogger(org.ow2.petals.microkernel.jbi.component.context.Constants.FRACTAL_COMPONENT_LOGGER_NAME));
        this.componentLoggerRBNames = new HashMap<String, String>();
        this.internalEndpoints = new HashSet<ServiceEndpoint>();
        this.externalEndpoints = new HashSet<ServiceEndpoint>();
    }

    public ServiceEndpoint activateEndpoint(QName serviceName, String endpointName)
            throws JBIException {
        this.logger.start();

        ParameterCheckHelper.isNullParameter(serviceName, "serviceName");
        ParameterCheckHelper.isNullParameter(serviceName, "endpointName");

        final QName service = serviceName;
        final String endpoint = endpointName;

        Document doc = this.getComponent().getServiceDescription(new ServiceEndpoint() {
            public DocumentFragment getAsReference(QName operationName) {
                return null;
            }

            public String getEndpointName() {
                return endpoint;
            }

            public QName[] getInterfaces() {
                return null;
            }

            public QName getServiceName() {
                return service;
            }

        });

        // switch the classloader to the container one
        // CAUTION : this must be done AFTER getServiceDescription is called
        ClassLoader componentClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        ServiceEndpoint se;

        List<QName> interfaces = new ArrayList<QName>(1);
        if (doc != null) {
            final List<QName> extractedInterfaces = this.getInterfacesFromComponentDescription(doc,
                    serviceName, endpointName);
            if (extractedInterfaces != null && extractedInterfaces.size() > 0) {
                interfaces.addAll(extractedInterfaces);
            } else {
                final QName defaultInterface = getDefaultInterface(serviceName);
                this.logger
                        .warning("No interface has been found in the component service description, set default to "
                                + defaultInterface);
                interfaces.add(defaultInterface);
            }
        } else {
            final QName defaultInterface = getDefaultInterface(serviceName);
            this.logger.warning("No description has been found for service '" + service.toString()
                    + "', endpoint name '" + endpointName + "'. Interface set default to "
                    + defaultInterface);
            interfaces.add(defaultInterface);
        }

        try {
            this.logger.debug("Activating the new created endpoint in the registry");
            se = this.componentContextCommunication.getEndpointRegistry().activateEndpoint(
                    serviceName, endpointName, interfaces, doc,
                    this.componentContextCommunication.getAddress());
        } finally {
            Thread.currentThread().setContextClassLoader(componentClassLoader);
        }
        this.internalEndpoints.add(se);
        this.logger.end();
        return se;
    }

    /**
     * Clear the component Context, remove objects dependent to the component
     * life cycle scope. To be used when to component is uninstalling.
     */
    public void clear() {
        this.deliveryChannel = null;
        this.jbiComponent = null;
    }

    private final static QName getDefaultInterface(final QName serviceName) {
        return new QName(serviceName.getNamespaceURI(), UNRESOLVED_INTERFACE
                + serviceName.getLocalPart());
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.component.ComponentContext#deactivateEndpoint(javax.jbi.servicedesc
     * .ServiceEndpoint)
     */
    public void deactivateEndpoint(javax.jbi.servicedesc.ServiceEndpoint endpoint)
            throws JBIException {
        this.logger.start();

        ParameterCheckHelper.isNullParameter(endpoint, "endpoint");

        // switch the classloader to the container one
        ClassLoader componentClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        try {
            this.componentContextCommunication.getEndpointRegistry().deactivateEndpoint(
                    endpoint.getEndpointName(), endpoint.getServiceName());
        } finally {
            Thread.currentThread().setContextClassLoader(componentClassLoader);
        }
        this.internalEndpoints.remove(endpoint);
        this.logger.end();
    }

    /**
     * deregister all endpoints (internal and external)
     * 
     */
    public void deregisterAllEndpoints() throws JBIException {
        this.logger.start();

        // switch the classloader to the container one
        ClassLoader componentClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        try {
            for (ServiceEndpoint se : this.internalEndpoints) {
                this.componentContextCommunication.getEndpointRegistry().deactivateEndpoint(
                        se.getEndpointName(), se.getServiceName());
            }

            for (ServiceEndpoint se : this.externalEndpoints) {
                this.componentContextCommunication.getEndpointRegistry()
                        .deregisterExternalEndpoint(se.getEndpointName(), se.getServiceName());
            }

        } finally {
            this.internalEndpoints.clear();
            this.externalEndpoints.clear();
            Thread.currentThread().setContextClassLoader(componentClassLoader);
        }

        this.logger.end();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.component.ComponentContext#deregisterExternalEndpoint(javax
     * .jbi.servicedesc.ServiceEndpoint)
     */
    public void deregisterExternalEndpoint(ServiceEndpoint externalEndpoint) throws JBIException {
        this.logger.start();

        ParameterCheckHelper.isNullParameter(externalEndpoint, "externalEndpoint");

        this.componentContextCommunication.getEndpointRegistry().deregisterExternalEndpoint(
                externalEndpoint.getEndpointName(), externalEndpoint.getServiceName());
        this.externalEndpoints.remove(externalEndpoint);
        this.logger.end();
    }

    public org.ow2.petals.microkernel.api.jbi.messaging.ServiceEndpoint getAddress() {
        return this.componentContextCommunication.getAddress();
    }

    public Component getComponent() {
        return this.jbiComponent;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.component.ComponentContext#getComponentName()
     */
    public String getComponentName() {
        return this.componentContextCommunication.getJBIDescriptor().getComponent()
                .getIdentification().getName();
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.component.ComponentContext#getDeliveryChannel()
     */
    public org.ow2.petals.microkernel.api.jbi.messaging.DeliveryChannel getDeliveryChannel()
            throws MessagingException {
        this.logger.start();

        if (this.deliveryChannel == null) {
            this.deliveryChannel = new DeliveryChannelImpl(this);
        } else if (!this.deliveryChannel.isOpened()) {
            this.deliveryChannel = new DeliveryChannelImpl(this);
        } else {
            // Do not throw an exception to be TCK and other containers
            // compliant
            // throw new MessagingException("Delivery Channel already open");
        }

        this.logger.end();
        return this.deliveryChannel;
    }

    /**
     * Return the actual deliveryChannel. Can be null.
     * 
     * @return
     */
    public DeliveryChannel getDeliveryChannelImpl() {
        return this.deliveryChannel;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.component.ComponentContext#getEndpoint(javax.xml.namespace.
     * QName, java.lang.String)
     */
    public ServiceEndpoint getEndpoint(QName service, String name) {
        this.logger.start();

        ParameterCheckHelper.isNullParameter(service, "service");
        ParameterCheckHelper.isNullParameter(name, "name");

        // switch the classloader to the container one
        ClassLoader componentClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        ServiceEndpoint result = null;
        try {
            result = this.componentContextCommunication.getEndpointRegistry().getEndpoint(service,
                    name);
        } catch (Exception e) {
            this.logger.warning(e.getMessage());
        } finally {
            Thread.currentThread().setContextClassLoader(componentClassLoader);
        }
        this.logger.end();
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.component.ComponentContext#getEndpointDescriptor(javax.jbi.
     * servicedesc.ServiceEndpoint)
     */
    public Document getEndpointDescriptor(ServiceEndpoint endpoint) throws JBIException {
        this.logger.start();
        Document result = null;
        ParameterCheckHelper.isNullParameter(endpoint, "endpoint");

        if (endpoint instanceof org.ow2.petals.microkernel.api.jbi.messaging.ServiceEndpoint) {
            this.logger.debug("Just get the description from the service endpoint");

            // FIXME = Do not get it but just check if it exists...
            if (this.getEndpoint(endpoint.getServiceName(), endpoint.getEndpointName()) != null) {

                // switch the classloader to the container one
                ClassLoader componentClassLoader = Thread.currentThread().getContextClassLoader();
                Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
                try {
                    result = ((org.ow2.petals.microkernel.api.jbi.messaging.ServiceEndpoint) endpoint)
                            .getDescription();
                } finally {
                    Thread.currentThread().setContextClassLoader(componentClassLoader);
                }

                if (result == null) {
                    this.logger
                            .debug("Can not find the service description in the service endpoint");
                }
            }
        } else {
            this.logger.debug("the Endpoint type must be "
                    + org.ow2.petals.microkernel.api.jbi.messaging.ServiceEndpoint.class
                            .getName());
        }
        this.logger.end();
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.component.ComponentContext#getEndpoints(javax.xml.namespace
     * .QName)
     */
    public ServiceEndpoint[] getEndpoints(QName interfaceName) {
        this.logger.start();

        // switch the classloader to the container one
        ClassLoader componentClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        ServiceEndpoint[] result = null;
        try {
            result = this.componentContextCommunication.getEndpointRegistry()
                    .getInternalEndpointsForInterface(interfaceName, LinkType.STANDARD);
        } catch (Exception e) {
            this.logger.warning(e.getMessage());
        } finally {
            Thread.currentThread().setContextClassLoader(componentClassLoader);
        }

        this.logger.end();
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.component.ComponentContext#getEndpointsForService(javax.xml
     * .namespace.QName)
     */
    public ServiceEndpoint[] getEndpointsForService(QName serviceName) {
        this.logger.start();

        ParameterCheckHelper.isNullParameter(serviceName, "serviceName");

        // switch the classloader to the container one
        ClassLoader componentClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        ServiceEndpoint[] result = null;
        try {
            result = this.componentContextCommunication.getEndpointRegistry()
                    .getInternalEndpointsForService(serviceName, LinkType.STANDARD);
        } catch (RegistryException e) {
            this.logger.warning(e.getMessage());
        } finally {
            Thread.currentThread().setContextClassLoader(componentClassLoader);
        }

        this.logger.end();
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.component.ComponentContext#getExternalEndpoints(javax.xml.namespace
     * .QName)
     */
    public ServiceEndpoint[] getExternalEndpoints(QName interfaceName) {
        this.logger.start();

        ParameterCheckHelper.isNullParameter(interfaceName, "interfaceName");

        // switch the classloader to the container one
        ClassLoader componentClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        ServiceEndpoint[] result = null;
        try {
            result = this.componentContextCommunication.getEndpointRegistry()
                    .getExternalEndpointsForInterface(interfaceName);
        } catch (RegistryException e) {
            this.logger.warning(e.getMessage());
        } finally {
            Thread.currentThread().setContextClassLoader(componentClassLoader);
        }

        this.logger.end();
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.component.ComponentContext#getExternalEndpointsForService(javax
     * .xml.namespace.QName)
     */
    public ServiceEndpoint[] getExternalEndpointsForService(QName serviceName) {
        this.logger.start();

        ParameterCheckHelper.isNullParameter(serviceName, "serviceName");

        // switch the classloader to the container one
        ClassLoader componentClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        ServiceEndpoint[] result = null;
        try {
            result = this.componentContextCommunication.getEndpointRegistry()
                    .getExternalEndpointsForService(serviceName);
        } catch (RegistryException e) {
            this.logger.warning(e.getMessage());
        } finally {
            Thread.currentThread().setContextClassLoader(componentClassLoader);
        }

        this.logger.end();
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.component.ComponentContext#getInstallRoot()
     */
    public String getInstallRoot() {
        this.logger.call();

        return this.componentContextCommunication.getInstallationRoot();
    }

    public Jbi getJbi() {
        return this.componentContextCommunication.getJBIDescriptor();
    }

    public Logger getLogger() {
        return this.componentContextCommunication.getLogger();
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.component.ComponentContext#getLogger(java.lang.String,
     * java.lang.String)
     */
    public Logger getLogger(String suffix, String resourceBundleName)
            throws MissingResourceException, JBIException {
        this.logger.start();

        String loggerName = this.getComponentName();

        // append the suffix to the logger Name
        if (suffix == null) {
            throw new IllegalArgumentException("suffix cannot be null");
        } else {
            if (suffix.length() > 0) {
                loggerName += "." + suffix;
            }
        }

        // get the corresponding logger
        Logger jdkLogger = this.getComponentLogger(loggerName, resourceBundleName);

        this.logger.end();
        return jdkLogger;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.component.ComponentContext#getMBeanNames()
     */
    public MBeanNames getMBeanNames() {
        this.logger.call();

        return this.componentContextCommunication.getAdminService().getMBeanNames();
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.component.ComponentContext#getMBeanServer()
     */
    public MBeanServer getMBeanServer() {
        this.logger.call();

        return this.componentContextCommunication.getAdminService().getMBeanServer();
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.component.ComponentContext#getNamingContext()
     */
    public InitialContext getNamingContext() {
        this.logger.call();

        return this.componentContextCommunication.getInitialContext();
    }

    public RouterService getRouter() {
        return this.componentContextCommunication.getRouterService();
    }

    /**
     * @return the securityContext
     */
    public SecurityContext getSecurityContext() {
        return this.securityContext;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.component.ComponentContext#getTransactionManager()
     */
    public Object getTransactionManager() {
        this.logger.call();

        // FIXME: Bug as the generated class of the Fractal binding to the
        // transaction manager is not in the classloader of the component when
        // this latest is isolated. It is not possible to cast it in the
        // component!
        Object result = this.componentContextCommunication.getTransactionManagerService();

        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.component.ComponentContext#getWorkspaceRoot()
     */
    public String getWorkspaceRoot() {
        this.logger.call();

        return this.componentContextCommunication.getWorkspaceRoot();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.component.ComponentContext#registerExternalEndpoint(javax.jbi
     * .servicedesc.ServiceEndpoint)
     */
    public void registerExternalEndpoint(ServiceEndpoint externalEndpoint) throws JBIException {
        this.logger.start();

        ParameterCheckHelper.isNullParameter(externalEndpoint, "externalEndpoint");

        // switch the classloader to the container one
        ClassLoader componentClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        try {
            this.componentContextCommunication.getEndpointRegistry().registerExternalEndpoint(
                    externalEndpoint);
        } finally {
            Thread.currentThread().setContextClassLoader(componentClassLoader);
        }
        this.externalEndpoints.add(externalEndpoint);

        this.logger.end();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.component.ComponentContext#resolveEndpointReference(org.w3c
     * .dom.DocumentFragment)
     */
    public ServiceEndpoint resolveEndpointReference(DocumentFragment epr) {
        this.logger.start();

        ParameterCheckHelper.isNullParameter(epr, "epr");

        javax.jbi.servicedesc.ServiceEndpoint result = null;

        // switch the classloader to the container one
        ClassLoader componentClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        try {

            for (Installer installer : this.componentContextCommunication.getAdminService()
                    .getInstallers()) {
                result = installer.getComponent().resolveEndpointReference(epr);
                if (result != null) {
                    break;
                }
            }
        } finally {
            Thread.currentThread().setContextClassLoader(componentClassLoader);
        }

        this.logger.end();
        return result;

    }

    public void setComponent(Component component) {
        this.jbiComponent = component;
    }

    public void setDeliveryChannel(DeliveryChannel deliveryChannel) {
        this.deliveryChannel = deliveryChannel;
    }

    /**
     * @param securityContext
     *            the securityContext to set
     */
    public void setSecurityContext(SecurityContext securityContext) {
        this.securityContext = securityContext;
    }

    /**
     * The method creates or retrieve a logger for the component.
     * 
     * @param loggerName
     * @param resourceBundleName
     * 
     * @return the jdk form logger
     * 
     * @throws JBIException
     */
    private Logger getComponentLogger(String loggerName, String resourceBundleName)
            throws JBIException {
        this.logger.start();

        /*
         * As specified in the JBI specification it's forbidden to call a same
         * logger with two different Resource bundle names
         */
        String rbName = this.componentLoggerRBNames.get(loggerName);
        if ((rbName != null) && !rbName.equals(resourceBundleName)) {
            throw new JBIException("Logger already exists for suffix " + loggerName + " but with "
                    + rbName + " as resource bundle instead of the expected " + resourceBundleName);
        }

        Logger jdkLogger = null;

        // get a new logger or retrieve it
        Logger newLogger = Logger.getLogger(COMPONENT_LOGGER_PREFIX + "." + loggerName,
                resourceBundleName);

        // if the logger already exits, return it directly
        if (newLogger.getHandlers().length == 0) {

            // add the handler to the new logger
            try {
                // TODO: aruffie: find a good means to remove fractal factory
                // and get handler
                // newLogger.addHandler(this.loggerFactory.getHandler("petalsConsole"));
                // newLogger.addHandler(this.loggerFactory.getHandler("petalsFile"));
            } catch (Exception e) {
                throw new JBIException(e);
            }

            // Add loggerName to registered rbNames for loggers
            this.componentLoggerRBNames.put(loggerName, resourceBundleName);
        }

        // cast the logger to fit the JDK Logger class
        jdkLogger = (Logger) newLogger;

        this.logger.end();
        return jdkLogger;
    }

    private List<QName> getInterfacesFromComponentDescription(Document doc, QName serviceName,
            String endpointName) {
        List<QName> result = new ArrayList<QName>();
        try {
            WSDL4ComplexWsdlReader wsdl4ComplexWsdlReader = null;
            try {
                wsdl4ComplexWsdlReader = WSDL4ComplexWsdlReaders.takeWSDL4ComplexWsdlReader();
                Description wsdlDescription = wsdl4ComplexWsdlReader.read(doc);

                if ((wsdlDescription != null) && (wsdlDescription.getServices() != null)) {
                    Service ss = wsdlDescription.getService(serviceName);
                    if (ss != null) {
                        if (ss.getEndpoint(endpointName) != null) {
                            if ((ss.getInterface() != null)
                                    && (ss.getInterface().getQName() != null)) {
                                result.add(ss.getInterface().getQName());
                            } else {
                                this.logger
                                        .warning("No interface for endpoint in component description for service '"
                                                + ss.getQName() + "'");
                            }
                        } else {
                            this.logger
                                    .warning("No endpoint in component description for service '"
                                            + serviceName + "'");
                        }
                    } else {
                        this.logger.warning("No such service in component description '"
                                + serviceName + "'");
                    }
                }
            } finally {
                if (wsdl4ComplexWsdlReader != null) {
                    WSDL4ComplexWsdlReaders.releaseWSDL4ComplexWsdlReader(wsdl4ComplexWsdlReader);
                }
            }
        } catch (WSDL4ComplexWsdlException e) {
            this.logger.warning(e.getMessage());
        } catch (WSDLException e) {
            this.logger.warning(e.getMessage());
        } catch (URISyntaxException e) {
            this.logger.warning(e.getMessage());
        }
        return result;
    }

}
