/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.task;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URL;
import java.net.URLDecoder;

import org.apache.commons.io.FileUtils;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.microkernel.api.jbi.management.Context;
import org.ow2.petals.microkernel.api.util.LoggingUtil;

/**
 * This task copy entity (component, shared lib, service assembly) package into the work directory.
 * <p>
 * Inputs : archive {@link URI} <br/>
 * Outputs : {@link File} pointing on unzipped archive <br/>
 * The task has a non-robust approach: it stops at the first subtask
 * failed and have no undo capability.
 * </p>
 * 
 * @author Olivier Fabre - EBM WebSourcing
 * @author Frederic Gardes - EBM WebSourcing
 */
public class CopyArchiveToWorkDirTask extends AbstractFileManipulationTask {

    public CopyArchiveToWorkDirTask(final LoggingUtil log,
            final ContainerConfiguration containerConfiguration) {
        super(log, containerConfiguration);
    }

    /*
     * (non-Javadoc)
     * @see org.ow2.petals.jbi.management.Task#execute(org.ow2.petals.jbi.management.Context)
     */
    public void execute(Context context) throws Exception {
        URL archiveURL = context.getArchiveUrl();
        File archive = this.copyEntityPackage(archiveURL);
        context.setArchiveUrl(archive.toURI().toURL());
    }

    @SuppressWarnings("deprecation")
    private File copyEntityPackage(URL archiveURL) throws PetalsException {
        File archiveFile = new File(URLDecoder.decode(archiveURL.getPath()));
        File destFile = new File(this.getWorkDirectory(), archiveFile.getName());
        try {
            FileUtils.copyFile(archiveFile, destFile);
        } catch (IOException e) {
            throw new PetalsException("Can't copy entity package to work directory", e);
        }
        
        return destFile;
    }

    /*
     * (non-Javadoc)
     * @see org.ow2.petals.jbi.management.Task#undo(org.ow2.petals.jbi.management.Context)
     */
    @SuppressWarnings("deprecation")
    public void undo(Context context) throws Exception {
        URL archiveURL = context.getArchiveUrl();
        if (archiveURL != null) {
            final String archiveFilepath = URLDecoder.decode(archiveURL.getPath());
            try {
                FileUtils.forceDelete(new File(archiveFilepath));
            } catch (IOException e) {
                this.log.warning("Failed to clean the copied archive '" + archiveFilepath
                        + "'");
            }
        }
    }
}
