/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.messaging.exchange;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.nio.charset.Charset;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessageExchangeFactory;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.transaction.SystemException;
import javax.xml.namespace.QName;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeWrapper;
import org.ow2.petals.microkernel.api.jbi.ComponentContext;
import org.ow2.petals.microkernel.api.jbi.messaging.DeliveryChannel;
import org.ow2.petals.microkernel.api.jbi.messaging.RoutingException;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.system.transaction.TransactionManagerImpl;

import com.ebmwebsourcing.easycommons.pooling.PoolException;
import com.ebmwebsourcing.easycommons.stream.InputStreamForker;
import com.ebmwebsourcing.easycommons.stream.ReaderInputStream;
import com.ebmwebsourcing.easycommons.xml.Transformers;

/**
 * @author Adrien LOUIS - EBM WebSourcing
 * @author Olivier Fabre - EBM WebSourcing
 * @author Roland Naudin - EBM WebSourcing
 */
public class DeliveryChannelImpl implements DeliveryChannel {

    private final ComponentContext componentContext;

    private final LoggingUtil log;

    private final Logger logger;

    protected boolean opened = true;

    /**
     * Constructor
     * 
     * @param context
     * @param router
     * @param logger
     */
    public DeliveryChannelImpl(final ComponentContext context) {
        this.logger = context.getLogger();

        this.log = new LoggingUtil(this.logger);

        this.componentContext = context;
    }

    /**
     * Check if the <code>MessageExchange</code> is Ok (not null and not
     * terminated)
     * 
     * @throws MessagingException
     */
    private static final void checkMessageExchange(final MessageExchangeWrapper exchange)
            throws MessagingException {
        if (exchange == null) {
            throw new MessagingException("MessageExchange is null");
        }

        final MessageExchangeWrapper exchangeDecorator = (MessageExchangeWrapper) exchange;

        if (exchangeDecorator.getMessageExchange().isTerminated()) {
            throw new MessagingException(
                    "MessageExchange is terminated, it can not be sent any more");
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.messaging.DeliveryChannel#accept()
     */
    public MessageExchange accept() throws MessagingException {
        this.log.call();
        final MessageExchange me = this.poll(-1);

        return me;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.messaging.DeliveryChannel#accept(long)
     */
    public MessageExchange accept(final long timeoutMS) throws MessagingException {
        this.log.call(timeoutMS);
        if (timeoutMS < 0) {
            throw new MessagingException("Timeout must be positive");
        }
        final MessageExchange me = this.poll(timeoutMS);
        return me;
    }

    /**
     * Check if the <code>DeliveryChannel</code> is opened
     * 
     * @throws MessagingException
     */
    public void checkDeliveryChannelIsOpened() throws MessagingException {
        if (!this.opened) {
            throw new MessagingException("DeliveryChannel is closed");
        }
    }

    /**
     * This method wake up all threads that are waiting for a message. This
     * method deregister all endpoints for this component.
     * 
     * @see javax.jbi.messaging.DeliveryChannel#close()
     */
    public synchronized void close() throws MessagingException {
        this.log.start();

        this.checkDeliveryChannelIsOpened();

        this.opened = false;

        this.log.end();
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.messaging.DeliveryChannel#createExchangeFactory()
     */
    public MessageExchangeFactory createExchangeFactory() {
        this.log.call();
        PersistedContextualMessageExchangeFactoryImpl exchangeFactoryImpl = PersistedContextualMessageExchangeFactoryImpl.createContextualMessageExchangeFactory(this.componentContext.getAddress(), this.logger);
        return exchangeFactoryImpl;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.messaging.DeliveryChannel#createExchangeFactory(javax.xml.namespace
     * .QName)
     */
    public MessageExchangeFactory createExchangeFactory(final QName interfaceName) {
        this.log.call(interfaceName);
        PersistedContextualMessageExchangeFactoryImpl exchangeFactoryImpl = PersistedContextualMessageExchangeFactoryImpl.createContextualMessageExchangeFactoryForInterface(this.componentContext.getAddress(), this.logger, interfaceName);
        return exchangeFactoryImpl;
    }

    /*
     * (non-Javadoc)
     * 
     * @seejavax.jbi.messaging.DeliveryChannel#createExchangeFactory(javax.jbi.
     * servicedesc.ServiceEndpoint)
     */
    public MessageExchangeFactory createExchangeFactory(final ServiceEndpoint endpoint) {
        this.log.call(endpoint);
        PersistedContextualMessageExchangeFactoryImpl exchangeFactoryImpl = PersistedContextualMessageExchangeFactoryImpl.createContextualMessageExchangeFactoryForEndpoint(this.componentContext.getAddress(), this.logger, endpoint);
        return exchangeFactoryImpl;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.messaging.DeliveryChannel#createExchangeFactoryForService(javax
     * .xml.namespace.QName)
     */
    public MessageExchangeFactory createExchangeFactoryForService(final QName serviceName) {
        this.log.call(serviceName);
        PersistedContextualMessageExchangeFactoryImpl exchangeFactoryImpl = PersistedContextualMessageExchangeFactoryImpl.createContextualMessageExchangeFactoryForService(this.componentContext.getAddress(), this.logger, serviceName);
        return exchangeFactoryImpl;
    }

    public boolean isOpened() {
        return this.opened;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.messaging.DeliveryChannel#send(javax.jbi.messaging.MessageExchange
     * )
     */
    public void send(final MessageExchange exchange) throws MessagingException {
        this.log.call("Exchange Id:" + exchange.getExchangeId());

        this.sendExchange((MessageExchangeWrapper) exchange, 0, false);
    }

    /*
     * (non-Javadoc)
     * 
     * @seejavax.jbi.messaging.DeliveryChannel#sendSync(javax.jbi.messaging.
     * MessageExchange)
     */
    public boolean sendSync(final MessageExchange exchange) throws MessagingException {
        return this.sendSync(exchange, 0);
    }

    /*
     * (non-Javadoc)
     * 
     * @seejavax.jbi.messaging.DeliveryChannel#sendSync(javax.jbi.messaging.
     * MessageExchange, long)
     */
    public boolean sendSync(final MessageExchange exchange, final long timeoutMS)
            throws MessagingException {
        this.log.call("Exchange Id:" + exchange.getExchangeId() + " - Timeout:" + timeoutMS);
        boolean isTimeout = false;

        // the exchange can be only a MessageExchangeDecorator
        final MessageExchangeWrapper exchangeDecorator = (MessageExchangeWrapper) exchange;

        try {
            this.sendExchange(exchangeDecorator, timeoutMS, true);
            if (exchangeDecorator.isTimeout()) {
                // reset the ownership of the ME
                // TODO do this in the NMR
                // reset the timeout
                exchangeDecorator.setTimeout(false);
                isTimeout = true;
            }
        } catch (final MessagingException e) {
            this.log.debug(e.getMessage());
            // get the ownership of the ME
            exchangeDecorator.setStatus(ExchangeStatus.ERROR);
            exchangeDecorator.getMessageExchange().setTerminated(true);

            // If the cause is an Interrupted exception, we return false
            Throwable causeException = e.getCause();
            while (causeException != null) {
                if (causeException instanceof InterruptedException) {
                    this.log.end();
                    return false;
                } else {
                    causeException = causeException.getCause();
                }
            }
            throw e;
        }

        this.log.end();

        return !isTimeout;
    }

    /**
     * Log the current message, it can be an Out, Fault or In message.
     * 
     * @param exchange a message exchange decorator
     * @throws JBIException
     * @throws TransformerException
     *             if an unrecoverable error occurs during the course of the
     *             transformation
     * @throws PoolException
     *             This method uses a transformer got from a transformer pool.
     *             This unchecked exception is thrown if the current thread
     *             waits for a transformer of the pool and is interrupted when
     *             the pool policy is WAIT or if there is no more available
     *             transformer in the transformer pool when the pool policy is
     *             REJECT
     */
    private void logMessage(final MessageExchangeWrapper exchange) {
        java.util.logging.Logger componentLogger = null;
        // dirty but will log to the component logger.
        try {
            componentLogger = this.componentContext.getLogger("", null);
            if (componentLogger.isLoggable(Level.FINEST)) {
                if (ExchangeStatus.ACTIVE.equals(exchange.getStatus())) {
                    NormalizedMessage normalizedMessage = exchange
                            .getMessage(MessageExchangeImpl.OUT_MSG);
                    String messageType = MessageExchangeImpl.OUT_MSG;
                    if (normalizedMessage == null) {
                        normalizedMessage = exchange.getFault();
                        messageType = "Fault";
                    }
                    if (normalizedMessage == null) {
                        normalizedMessage = exchange.getMessage(MessageExchangeImpl.IN_MSG);
                        messageType = MessageExchangeImpl.IN_MSG;
                    }

                    componentLogger.finest("Message " + messageType + " with Id '"
                            + exchange.getExchangeId() + "' to send");
                    componentLogger.finest("Operation : " + exchange.getOperation());
                    componentLogger.finest("MEP : " + exchange.getPattern());

                    for (final Object propertyName : exchange.getPropertyNames()) {
                        componentLogger.finest("Exchange property '" + propertyName + "' : "
                                + exchange.getProperty((String) propertyName));
                    }

                    for (final Object propertyName : normalizedMessage.getPropertyNames()) {
                        componentLogger.finest("Message property '" + propertyName + "' : "
                                + normalizedMessage.getProperty((String) propertyName));
                    }

                    Source content = normalizedMessage.getContent();
                    if (content != null) {
                        if (content instanceof StreamSource) {
                            InputStreamForker streamForker;
                            final StreamSource streamContent = (StreamSource) content;
                            final InputStream isContent = streamContent.getInputStream();
                            if (isContent != null) {
                                // The StreamSource was created from an
                                // InputStream
                                streamForker = new InputStreamForker(isContent);
                            } else {
                                // The StreamSource was created from a Reader
                                // we wrap it as an InputStream
                                streamForker = new InputStreamForker(new ReaderInputStream(
                                        streamContent.getReader()));
                            }
                            normalizedMessage.setContent(new StreamSource(streamForker.fork()));
                            content = new StreamSource(streamForker.fork());
                        }

                        Transformer transformer = Transformers.takeTransformer();
                        transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "no");
                        transformer.setOutputProperty(OutputKeys.METHOD, "xml");
                        transformer.setOutputProperty(OutputKeys.ENCODING, Charset.defaultCharset()
                                .name());
                        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
                        try {
                            transformer.transform(content, new StreamResult(byteArrayOutputStream));
                        } finally {
                            Transformers.releaseTransformer(transformer);
                        }
                        componentLogger.finest("Message Content : "
                                + new String(byteArrayOutputStream.toByteArray()));
                    } else {
                        componentLogger.finest("Message Content is null");
                    }

                    for (final Object attachmentName : normalizedMessage.getAttachmentNames()) {
                        componentLogger.finest("Got a message attachment named '" + attachmentName
                                + "'");
                    }
                } else if (ExchangeStatus.DONE.equals(exchange.getStatus())) {
                    componentLogger.finest("Message acknowledgement DONE with Id '"
                            + exchange.getExchangeId() + "' to send");
                } else {
                    componentLogger.finest("Message acknowledgement ERROR with Id '"
                            + exchange.getExchangeId() + "' to send");
                    componentLogger.finest("Message Error : " + exchange.getError().getMessage());
                }
            }
        } catch (final Exception e) {
            if (componentLogger != null) {
                componentLogger.fine("Failed to log the content of the message to send: "
                        + e.getMessage());
            }
        }
    }

    /**
     * Poll for a receipt message exchange.
     * 
     * @param timeoutMS
     * 
     * @return The received message exchange, or {@code null} if timeout
     *         occurred
     * 
     * @throws MessagingException
     */
    private MessageExchange poll(final long timeoutMS) throws MessagingException {
        this.log.start();

        this.checkDeliveryChannelIsOpened();

        MessageExchangeWrapper messageDecorator = null;
        try {
            messageDecorator = this.componentContext.getRouter().receive(this.componentContext,
                    timeoutMS);
        } catch (final RoutingException e) {
            throw new MessagingException(e);
        }

        if (messageDecorator != null && this.componentContext.getTransactionManager() != null) {
            // decode the transaction to the Message Exchange if it is
            // transacted
            TransactionManagerImpl.decodeTransactionContext(messageDecorator);
        }

        this.log.end();

        return messageDecorator;
    }

    /**
     * Send the exchange to the underlying layer.<br>
     * Unused data in the exchange are cleaned before.<br>
     * If the message to exchange is a mere status information, and the exchange
     * is set to not send acknowledgment, the message is not sent.<br>
     */
    private void sendExchange(final MessageExchangeWrapper exchangeDecorator, final long timeout,
            final boolean sync) throws MessagingException {
        this.checkDeliveryChannelIsOpened();

        DeliveryChannelImpl.checkMessageExchange(exchangeDecorator);

        this.logMessage(exchangeDecorator);

        // switch the classloader to the container one
        final ClassLoader componentClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        try {
            // check if the exchange is terminated
            if (!ExchangeStatus.ACTIVE.equals(exchangeDecorator.getStatus())) {
                exchangeDecorator.setTerminated(true);
                if (sync) {
                    throw new MessagingException(
                            "Sending an acknowledgment in synchronous mode is forbidden");
                }
            }

            // encode the transaction to the Message Exchange if it is
            // transacted
            if (this.componentContext.getTransactionManager() != null) {
                TransactionManagerImpl.encodeTransactionContext(exchangeDecorator);
            }

            if (sync) {
                this.componentContext.getRouter().sendSync(this.componentContext,
                        exchangeDecorator, timeout);
                if (!exchangeDecorator.isTimeout()
                        && this.componentContext.getTransactionManager() != null) {
                    // decode the transaction to the Message Exchange if it is
                    // transacted
                    TransactionManagerImpl.decodeTransactionContext(exchangeDecorator);
                }
            } else {
                this.componentContext.getRouter().send(this.componentContext, exchangeDecorator);
            }

        } catch (final JBIException e) {
            throw new MessagingException(e);
        } catch (final SystemException e) {
            throw new MessagingException(e);
        } finally {
            Thread.currentThread().setContextClassLoader(componentClassLoader);
        }

        this.log.end();
    }
}
