/**
 * Copyright (c) 2008-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.messaging.routing.module;

import java.io.IOException;
import java.net.URISyntaxException;
import java.net.URL;
import java.security.Principal;
import java.util.ArrayList;
import java.util.Map;
import java.util.Set;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange.Role;
import javax.security.auth.Subject;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.ow2.petals.jaas.GroupPrincipal;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeWrapper;
import org.ow2.petals.microkernel.api.jbi.ComponentContext;
import org.ow2.petals.microkernel.api.jbi.messaging.RoutingException;
import org.ow2.petals.microkernel.api.jbi.messaging.ServiceEndpoint;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.jbi.security.AuthorizationEntry;
import org.ow2.petals.microkernel.jbi.security.AuthorizationException;
import org.ow2.petals.microkernel.jbi.security.AuthorizationMap;
import org.ow2.petals.microkernel.jbi.security.DefaultAuthorizationMap;
import org.ow2.petals.microkernel.jbi.security.DefaultAuthorizationParser;
import org.ow2.petals.microkernel.transport.util.TransportSendContext;

/**
 * A router module for authorizing routing.
 * <p>
 * The accessible endpoint/service/operation are defined as accessible in a configuration file
 * (authorization.cfg). This file must be available in the petals
 * classpath(generally under the conf directory). The incoming message must
 * contains a security subject where principals match with the defined ones.
 * </p>
 * 
 * @author Christophe HAMERLING - EBM WebSourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = SenderModule.class))
public class AuthorizationModule implements SenderModule {

    /**
     * Logger wrapper.
     */
    protected LoggingUtil log;

    public static final String CONFIGURATION = "authorization.cfg";

    private AuthorizationMap authorizationMap;

    /**
     * Start the Fractal component
     * 
     * @throws IOException
     * @throws URISyntaxException
     * @throws AuthorizationException
     */
    @LifeCycle(on = LifeCycleType.START)
    protected void start() throws IOException, URISyntaxException, AuthorizationException {
        this.log = new LoggingUtil(java.util.logging.Logger.getLogger(Constants.FRACTAL_COMPONENT_LOGGER_NAME_AUTHORIZATION));
        this.log.call();

        final URL authURL = this.getClass().getResource("/" + AuthorizationModule.CONFIGURATION);
        if (authURL == null) {
            IOException e = new IOException(
                    "Can not load the authorization resource from classpath");
            this.log.error("Failed to start Authorization Module", e);
            throw e;
        }

        try {
            final DefaultAuthorizationParser parser = new DefaultAuthorizationParser(authURL
                    .toURI());
            this.authorizationMap = parser.parse();
            if (this.authorizationMap == null) {
                this.authorizationMap = new DefaultAuthorizationMap(
                        new ArrayList<AuthorizationEntry>(0));
            }
        } catch (final URISyntaxException e) {
            this.log.error("Failed to start Authorization Module", e);
            throw e;
        } catch (final AuthorizationException e) {
            this.log.error("Failed to start Authorization Module", e);
            throw e;
        }
    }

    /**
     * Stop the Fractal component
     */
    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() throws RoutingException {
        this.log.call();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.messaging.routing.module.SenderModule#electEndpoints
     * (java.util.Map,
     * org.ow2.petals.jbi.component.context.ComponentContextImpl,
     * org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl)
     */
    public void electEndpoints(final Map<ServiceEndpoint, TransportSendContext> electedEndpoints,
            final ComponentContext sourceComponentContext,
            final MessageExchangeWrapper exchangeDecorator) throws RoutingException {
        this.authorizeSend(exchangeDecorator);
    }

    /**
     * Authorize to send the message. This authorization is based on the message
     * exchange subject. If the subject principals match with the local
     * authorization map, the message can be send to the foreign
     * service/operation.
     * 
     * @param source
     * @param exchange
     * @param timeOut
     * @throws RoutingException
     *             if the subject is null or if the user is not authorized to
     *             send message to the JBI service (ie not found in
     *             authorization map).
     */
    protected void authorizeSend(MessageExchangeWrapper exchange) throws RoutingException {
        if ((exchange.getRole() == Role.PROVIDER) && (exchange.getStatus() != ExchangeStatus.DONE)) {
            final javax.jbi.servicedesc.ServiceEndpoint se = exchange.getEndpoint();
            if (se != null) {
                final Set<Principal> acls = this.authorizationMap.getAccessControlList(se, exchange
                        .getOperation());

                if (this.log.isDebugEnabled()) {
                    for (Principal principal : acls) {
                        this.log.debug("Server defined ACL for operation : "
                                + exchange.getOperation().toString());
                        this.log.debug(principal.getName() + "(" + principal.getClass().getName()
                                + ")");
                    }
                }

                // skip message if ACL contains ALL policy
                if (!acls.contains(GroupPrincipal.ALL)) {
                    final Subject subject = exchange.getMessage("in").getSecuritySubject();
                    if (subject == null) {
                        throw new SecurityException(
                                "User not authenticated (security subject is null)");
                    }

                    acls.retainAll(subject.getPrincipals());
                    this.log.debug("ACLS size after retain is : " + acls.size());

                    if (acls.size() == 0) {
                        throw new SecurityException("Endpoint '" + se.getEndpointName()
                                + "' / Operation '" + exchange.getOperation()
                                + "' is not authorized for this user");
                    }
                }
            }
        }
    }
}
