/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.messaging.routing.monitoring;

import java.io.IOException;
import java.io.StringWriter;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Vector;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.xml.namespace.QName;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.stream.StreamResult;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeWrapper;
import org.ow2.petals.jmx.commons.PetalsMonitoringServiceConstants.Exchange;
import org.ow2.petals.jmx.commons.PetalsMonitoringServiceConstants.MEP;
import org.ow2.petals.jmx.commons.PetalsMonitoringServiceConstants.NormalizedMessage;
import org.ow2.petals.jmx.commons.PetalsMonitoringServiceConstants.Role;
import org.ow2.petals.jmx.commons.PetalsMonitoringServiceConstants.RouterMonitor;
import org.ow2.petals.jmx.commons.PetalsMonitoringServiceConstants.Status;
import org.ow2.petals.microkernel.api.jbi.ComponentContext;
import org.ow2.petals.microkernel.api.jbi.messaging.RoutingException;
import org.ow2.petals.microkernel.api.jbi.messaging.ServiceEndpoint;
import org.ow2.petals.microkernel.api.system.persistence.PersistenceService;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.jbi.messaging.routing.module.SenderModule;
import org.ow2.petals.microkernel.jbi.messaging.routing.util.SourcesForkerUtil;
import org.ow2.petals.microkernel.transport.util.TransportSendContext;

import com.ebmwebsourcing.easycommons.pooling.PoolException;
import com.ebmwebsourcing.easycommons.xml.Transformers;

/**
 * The NMR monitor service.
 * @author Roland Naudin - EBM WebSourcing
 */
@FractalComponent
@Provides(interfaces = { @Interface(name = "service", signature = RouterMonitorService.class),
        @Interface(name = "sendermodule", signature = SenderModule.class) })
public class RouterMonitorServiceImpl implements RouterMonitorService, SenderModule {

    /**
     * Logger wrapper.
     */
    private LoggingUtil log;

    /**
     * The Persistence Fractal Service
     */
    @Requires(name = "persistence")
    private PersistenceService persistenceService;

    /**
     * The Router Monitor filters.
     */
    private List<RouterMonitorFilter> routerMonitorFilters;

    /**
     * Create a string from a given XML Source.
     * 
     * @param source a XML source
     * 
     * @return the resulting string
     * 
     * @throws TransformerException
     *             if an unrecoverable error occurs during the course of the
     *             transformation
     * @throws PoolException
     *             This method uses a transformer got from a transformer pool.
     *             This unchecked exception is thrown if the current thread
     *             waits for a transformer of the pool and is interrupted when
     *             the pool policy is WAIT or if there is no more available
     *             transformer in the transformer pool when the pool policy is
     *             REJECT
     */
    private static final String createStringFromSource(final Source source)
            throws TransformerException {
        StringWriter buffer = new StringWriter();
        Result sresult = new StreamResult(buffer);
        
        Transformer transformer = Transformers.takeTransformer();
        try {
            transformer.transform(source, sresult);
        } finally {
            Transformers.releaseTransformer(transformer);
        }

        return buffer.toString();
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.messaging.routing.monitoring.RouterMonitorMBean#
     * clearMonitorStorage()
     */
    public void clearMonitorStorage() {
        this.persistenceService.clearExchangeStorage();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.messaging.routing.module.SenderModule#electEndpoints
     * (java.util.Map, org.ow2.petals.jbi.component.context.ComponentContext,
     * org.ow2.petals.jbi.messaging.exchange.MessageExchangeDecorator)
     */
    public void electEndpoints(Map<ServiceEndpoint, TransportSendContext> electedEndpoints,
            ComponentContext sourceComponentContext, MessageExchangeWrapper exchangeDecorator)
            throws RoutingException {
        this.log.call();

        if (MessageExchange.Role.CONSUMER.equals(exchangeDecorator.getRole())
                && exchangeDecorator.getMessage(MessageExchangeImpl.IN_MSG) != null) {
            checkMonitored(exchangeDecorator);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.messaging.routing.monitoring.RouterMonitorService#
     * exchangeReceived
     * (org.ow2.petals.jbi.messaging.exchange.MessageExchangeDecorator)
     */
    public void exchangeReceived(MessageExchangeWrapper exchangeDecorator) {
        this.log.call("Exchange Id : " + exchangeDecorator.getExchangeId());

        try {
            if (exchangeDecorator.getMessageExchange().isMonitored()
                    || exchangeDecorator.getMessageExchange().isPersisted()) {

                if (((ServiceEndpoint)exchangeDecorator
                        .getConsumerEndpoint())
                        .getLocation()
                        .getContainerName()
                        .equals(
((org.ow2.petals.launcher.api.service.ServiceEndpoint) exchangeDecorator
                                        .getEndpoint()).getLocation().getContainerName())) {
                    // local send, update just the receipt timestamp
                    this.persistenceService.updateMessageExchange(exchangeDecorator
                            .getMessageExchange(), false, Boolean.FALSE);
                } else {
                    try {
                        if (exchangeDecorator.getMessageExchange().isPersisted()) {
                            // fork the sources to reset them once persisted
                            SourcesForkerUtil.forkExchangeSources(exchangeDecorator);
                        }
                        if (MessageExchange.Role.PROVIDER.equals(exchangeDecorator.getRole())
                                && exchangeDecorator.getMessage(MessageExchangeImpl.IN_MSG) != null) {
                            // first time the exchange is add
                            this.persistenceService.addMessageExchange(exchangeDecorator
                                    .getMessageExchange(), !exchangeDecorator.getMessageExchange()
                                    .isMonitored(), Boolean.FALSE);
                        } else {
                            this.persistenceService.updateMessageExchange(exchangeDecorator
                                    .getMessageExchange(), !exchangeDecorator.getMessageExchange()
                                    .isMonitored(), Boolean.FALSE);
                        }
                    } finally {
                        if (exchangeDecorator.getMessageExchange().isPersisted()) {
                            // reset the sources
                            SourcesForkerUtil.forkExchangeSources(exchangeDecorator);
                            SourcesForkerUtil.cleanExchangeSources(exchangeDecorator);
                        }
                    }
                }
            }
        } catch (SQLException e) {
            this.log.warning("Failed to store a monitored exchange with Id '"
                    + exchangeDecorator.getExchangeId() + "'", e);
        } catch (IOException e) {
            this.log.warning("Failed to store a monitored exchange with Id '"
                    + exchangeDecorator.getExchangeId() + "'", e);
        } catch (MessagingException e) {
            this.log.warning("Failed to store a monitored exchange with Id '"
                    + exchangeDecorator.getExchangeId() + "'", e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.messaging.routing.monitoring.RouterMonitorService#
     * exchangeSent
     * (org.ow2.petals.jbi.messaging.exchange.MessageExchangeDecorator)
     */
    public void exchangeSent(MessageExchangeWrapper exchangeDecorator) {
        this.log.call("Exchange Id : " + exchangeDecorator.getExchangeId());

        try {
            if (exchangeDecorator.getMessageExchange().isMonitored()
                    || exchangeDecorator.getMessageExchange().isPersisted()) {
                if (exchangeDecorator.getMessageExchange().isPersisted()
                        && !((ServiceEndpoint)exchangeDecorator
                                .getConsumerEndpoint())
                                .getLocation()
                                .getContainerName()
                                .equals(
((org.ow2.petals.launcher.api.service.ServiceEndpoint) exchangeDecorator
                                                .getEndpoint()).getLocation().getContainerName())) {
                    // not local send, fork the sources has the source streams
                    // has already been sent
                    SourcesForkerUtil.forkExchangeSources(exchangeDecorator);
                }

                try {
                    if (MessageExchange.Role.PROVIDER.equals(exchangeDecorator.getRole())
                            && exchangeDecorator.getMessage(MessageExchangeImpl.IN_MSG) != null) {
                        // first time the exchange is add
                        this.persistenceService.addMessageExchange(exchangeDecorator
                                .getMessageExchange(), !exchangeDecorator.getMessageExchange()
                                .isMonitored(), Boolean.TRUE);
                    } else {
                        this.persistenceService.updateMessageExchange(exchangeDecorator
                                .getMessageExchange(), !exchangeDecorator.getMessageExchange()
                                .isMonitored(), Boolean.TRUE);
                    }
                } finally {
                    if (exchangeDecorator.getMessageExchange().isPersisted()) {
                        // reset the sources
                        SourcesForkerUtil.forkExchangeSources(exchangeDecorator);
                        if (MessageExchange.Role.CONSUMER.equals(exchangeDecorator.getRole())) {
                            SourcesForkerUtil.cleanExchangeSources(exchangeDecorator);
                        } // otherwise let the router clean the forked source
                        // streams
                    }
                }

            }
        } catch (SQLException e) {
            this.log.warning("Failed to store a monitored exchange with Id '"
                    + exchangeDecorator.getExchangeId() + "'", e);
        } catch (IOException e) {
            this.log.warning("Failed to store a monitored exchange with Id '"
                    + exchangeDecorator.getExchangeId() + "'", e);
        } catch (MessagingException e) {
            this.log.warning("Failed to store a monitored exchange with Id '"
                    + exchangeDecorator.getExchangeId() + "'", e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.messaging.routing.monitoring.RouterMonitorMBean#
     * getExchange(java.lang.String)
     */
    public Map<String, String> getExchange(String exchangeId) throws MessagingException {
        this.log.call();
        Map<String, String> result = null;
        try {
            MessageExchangeImpl exchange = this.persistenceService.getMessageExchange(exchangeId);
            if (exchange != null) {
                result = new HashMap<String, String>();
                result.put(Exchange.ID, exchange.getExchangeId());
                result.put(Exchange.PROVIDER_COMPONENT, ((ServiceEndpoint) exchange.getEndpoint())
                        .getLocation().getComponentName());
                result.put(Exchange.PROVIDER_CONTAINER, ((ServiceEndpoint) exchange.getEndpoint())
                        .getLocation().getContainerName());

                ServiceEndpoint consumerEndpoint = (ServiceEndpoint) exchange.getConsumerEndpoint();
                if (exchange.getConsumerEndpoint() != null) {
                    result.put(Exchange.CONSUMER_COMPONENT, 
                            consumerEndpoint.getLocation().getComponentName());
                    result.put(Exchange.CONSUMER_CONTAINER, 
                            consumerEndpoint.getLocation().getContainerName());
                }

                result.put(Exchange.INTERFACE_NAME, exchange.getInterfaceName().toString());
                result.put(Exchange.SERVICE_NAME, exchange.getService().toString());
                result.put(Exchange.ENDPOINT_NAME, ((ServiceEndpoint) exchange.getEndpoint())
                        .getEndpointName());
                result.put(Exchange.OPERATION_NAME, exchange.getOperation().toString());
                if (MessageExchange.Role.CONSUMER.equals(exchange.getRole())) {
                    result.put(Exchange.ROLE, Role.CONSUMER);
                } else {
                    result.put(Exchange.ROLE, Role.PROVIDER);
                }

                if (MessageExchangeImpl.IN_ONLY_PATTERN.equals(exchange.getPattern())) {
                    result.put(Exchange.MEP, MEP.IN_ONLY);
                } else if (MessageExchangeImpl.ROBUST_IN_ONLY_PATTERN.equals(exchange.getPattern())) {
                    result.put(Exchange.MEP, MEP.ROBUST_IN_ONLY);
                } else if (MessageExchangeImpl.IN_OUT_PATTERN.equals(exchange.getPattern())) {
                    result.put(Exchange.MEP, MEP.IN_OUT);
                } else if (MessageExchangeImpl.IN_OPTIONAL_OUT_PATTERN
                        .equals(exchange.getPattern())) {
                    result.put(Exchange.MEP, MEP.IN_OPTIONAL_OUT);
                }

                if (exchange.getStatus() != null) {
                    if (ExchangeStatus.ACTIVE.equals(exchange.getStatus())) {
                        result.put(Exchange.STATUS, Status.ACTIVE);
                    } else if (ExchangeStatus.DONE.equals(exchange.getStatus())) {
                        result.put(Exchange.STATUS, Status.DONE);
                    } else {
                        result.put(Exchange.STATUS, Status.ERROR);
                    }
                }

                if (exchange.getError() != null) {
                    result.put(Exchange.EXCEPTION, exchange.getError().getMessage());
                }

                if (exchange.isPersisted()) {
                    result.put(Exchange.IS_CONTENT_STORED, Boolean.TRUE.toString());
                } else {
                    result.put(Exchange.IS_CONTENT_STORED, Boolean.FALSE.toString());
                }

                for (Object key : exchange.getPropertyNames()) {
                    result.put(Exchange.PROPERTY_KEY_PREFIX + (String) key, exchange.getProperty(
                            (String) key).toString());
                }

                for (Entry<String, javax.jbi.messaging.NormalizedMessage> entry : exchange
                        .getMessages().entrySet()) {

                    for (Object key : entry.getValue().getPropertyNames()) {
                        result.put(NormalizedMessage.NORMALIZED_MESSAGE_PREFIX + entry.getKey()
                                + "-" + NormalizedMessage.PROPERTY_KEY_PREFIX + (String) key, entry
                                .getValue().getProperty((String) key).toString());
                    }

                    try {
                        result.put(NormalizedMessage.NORMALIZED_MESSAGE_PREFIX + entry.getKey()
                                + "-" + NormalizedMessage.CONTENT, createStringFromSource(entry
                                .getValue().getContent()));
                    } catch (TransformerException e) {
                        result.put(NormalizedMessage.CONTENT, e.getMessage());
                    } catch (PoolException e) {
                        result.put(NormalizedMessage.CONTENT, e.getMessage());
                    }

                    for (Object key : entry.getValue().getAttachmentNames()) {
                        result.put(NormalizedMessage.NORMALIZED_MESSAGE_PREFIX + entry.getKey()
                                + "-" + NormalizedMessage.ATTACHMENT_KEY_PREFIX + (String) key,
                                entry.getValue().getAttachment((String) key).toString());
                    }
                }

            }
        } catch (SQLException e) {
            throw new MessagingException(e);
        } catch (IOException e) {
            throw new MessagingException(e);
        } catch (ClassNotFoundException e) {
            throw new MessagingException(e);
        }
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.messaging.routing.monitoring.RouterMonitorMBean#
     * getExchangeHistory(java.lang.String)
     */
    public Map<String, Long> getExchangeHistory(String exchangeId) throws MessagingException {
        final Map<String, Long> result;

        try {
            result = this.persistenceService.getMessageExchangeHistory(exchangeId);
        } catch (SQLException e) {
            throw new MessagingException(e);
        }

        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.messaging.routing.monitoring.RouterMonitorMBean#
     * getExchangeIds(javax.xml.namespace.QName, javax.xml.namespace.QName,
     * java.lang.String, javax.xml.namespace.QName, long, long)
     */
    public List<String> getExchangeIds(QName interfaceName, QName serviceName, String endpointName,
            QName operationName, long startDate, long endDate) throws MessagingException {
        this.log.call();
        final List<String> result;

        try {
            result = this.persistenceService.getMessageExchangeIds(interfaceName, serviceName,
                    endpointName, operationName, startDate, endDate);
        } catch (SQLException e) {
            throw new MessagingException(e);
        }

        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.messaging.routing.monitoring.RouterMonitorMBean#
     * getExchangeIds(java.lang.String, java.lang.String)
     */
    public List<String> getExchangeIds(String propertyKey, String propertyValue)
            throws MessagingException {
        this.log.call();
        final List<String> result;

        try {
            result = this.persistenceService.getMessageExchangeIds(propertyKey, propertyValue);
        } catch (SQLException e) {
            throw new MessagingException(e);
        } catch (IOException e) {
            throw new MessagingException(e);
        }

        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.messaging.routing.monitoring.RouterMonitorMBean#
     * getExchanges(javax.xml.namespace.QName, javax.xml.namespace.QName,
     * java.lang.String, javax.xml.namespace.QName, java.util.Date,
     * java.util.Date)
     */
    public int getExchanges(QName interfaceName, QName serviceName, String endpointName,
            QName operationName, long startDate, long endDate) throws MessagingException {
        this.log.call();
        int result;

        try {
            result = this.persistenceService.getMessageExchanges(interfaceName, serviceName,
                    endpointName, operationName, startDate, endDate);
        } catch (SQLException e) {
            throw new MessagingException(e);
        }

        return result;

    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.messaging.routing.monitoring.RouterMonitorMBean#
     * getMonitoredExchanges()
     */
    public List<Map<String, String>> getMonitoredExchanges() {
        this.log.call();

        List<Map<String, String>> result = new ArrayList<Map<String, String>>();

        for (RouterMonitorFilter routerMonitorFilter : this.routerMonitorFilters) {
            Map<String, String> routerMonitor = new HashMap<String, String>();
            if (routerMonitorFilter.getInterfaceName() != null) {
                routerMonitor.put(RouterMonitor.INTERFACE_NAME, routerMonitorFilter
                        .getInterfaceName().toString());
            } else {
                routerMonitor.put(RouterMonitor.INTERFACE_NAME, "null");
            }
            if (routerMonitorFilter.getServiceName() != null) {
                routerMonitor.put(RouterMonitor.SERVICE_NAME, routerMonitorFilter.getServiceName()
                        .toString());
            } else {
                routerMonitor.put(RouterMonitor.SERVICE_NAME, "null");
            }
            routerMonitor.put(RouterMonitor.ENDPOINT_NAME, routerMonitorFilter.getEndpointName());
            if (routerMonitorFilter.getOperationName() != null) {
                routerMonitor.put(RouterMonitor.OPERATION_NAME, routerMonitorFilter
                        .getOperationName().toString());
            } else {
                routerMonitor.put(RouterMonitor.OPERATION_NAME, "null");
            }
            routerMonitor.put(RouterMonitor.STORE_EXCHANGES, Boolean.toString(routerMonitorFilter
                    .isStoreExchanges()));
            result.add(routerMonitor);
        }

        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.messaging.routing.monitoring.RouterMonitorMBean#
     * getMonitorStorageDuration()
     */
    public long getMonitorStorageDuration() {
        return this.persistenceService.getExchangeStorageDuration();
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.messaging.routing.monitoring.RouterMonitorMBean#
     * monitorExchanges(javax.xml.namespace.QName, javax.xml.namespace.QName,
     * java.lang.String, javax.xml.namespace.QName, boolean)
     */
    public void monitorExchanges(QName interfaceName, QName serviceName, String endpointName,
            QName operationName, boolean storeExchanges) throws MessagingException {
        this.log.call();
        final RouterMonitorFilter routerMonitorFilter = new RouterMonitorFilter(interfaceName,
                serviceName, endpointName, operationName, storeExchanges);
        if (this.routerMonitorFilters.contains(routerMonitorFilter)) {
            throw new MessagingException("The Router Monitor filter " + routerMonitorFilter
                    + " already exist");
        } else {
            this.routerMonitorFilters.add(routerMonitorFilter);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.messaging.routing.monitoring.RouterMonitorMBean#
     * setMonitorStorageDuration(long)
     */
    public void setMonitorStorageDuration(long duration) {
        this.persistenceService.setExchangeStorageDuration(duration);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.jbi.messaging.routing.monitoring.RouterMonitorMBean#
     * unmonitorExchanges(javax.xml.namespace.QName, javax.xml.namespace.QName,
     * java.lang.String, javax.xml.namespace.QName, boolean)
     */
    public void unmonitorExchanges(QName interfaceName, QName serviceName, String endpointName,
            QName operationName) throws MessagingException {
        final RouterMonitorFilter routerMonitorFilter = new RouterMonitorFilter(interfaceName,
                serviceName, endpointName, operationName, true);

        if (!this.routerMonitorFilters.remove(routerMonitorFilter)) {
            throw new MessagingException("The Router Monitor filter " + routerMonitorFilter
                    + " is not registered");
        }
    }

    /**
     * Start the Fractal component
     * 
     * @throws RoutingException
     */
    @LifeCycle(on = LifeCycleType.START)
    protected void start() {
        this.log = new LoggingUtil(java.util.logging.Logger.getLogger(Constants.FRACTAL_COMPONENT_LOGGER_NAME));
        this.log.start();

        this.routerMonitorFilters = new Vector<RouterMonitorFilter>();

        this.log.end();
    }

    /**
     * Stop the Fractal component
     */
    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() {
        this.log.call();
    }

    /**
     * Check if the exchange is monitored.
     * 
     * @param exchangeDecorator
     * 
     */
    private final void checkMonitored(MessageExchangeWrapper exchangeDecorator) {

        // first time the exchange is sent
        for (RouterMonitorFilter routerMonitorFilter : this.routerMonitorFilters) {

            String endpointName = null;
            if (exchangeDecorator.getEndpoint() != null) {
                endpointName = exchangeDecorator.getEndpoint().getEndpointName();
            }
            QName interfaceName = exchangeDecorator.getInterfaceName();
            QName serviceName = exchangeDecorator.getService();
            QName operationName = exchangeDecorator.getOperation();

            if ((routerMonitorFilter.getInterfaceName() == null || (interfaceName != null && interfaceName
                    .equals(routerMonitorFilter.getInterfaceName())))
                    && (routerMonitorFilter.getServiceName() == null || (serviceName != null && serviceName
                            .equals(routerMonitorFilter.getServiceName())))
                    && (routerMonitorFilter.getEndpointName() == null || (endpointName != null && endpointName
                            .equals(routerMonitorFilter.getEndpointName())))
                    && (routerMonitorFilter.getOperationName() == null || (operationName != null && operationName
                            .equals(routerMonitorFilter.getOperationName())))) {
                if (routerMonitorFilter.isStoreExchanges()) {
                    exchangeDecorator.getMessageExchange().setPersisted(true);
                } else {
                    exchangeDecorator.getMessageExchange().setMonitored(true);
                }
                break;
            }
        }
    }

}
