/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.messaging.routing.util;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;

import org.ow2.petals.jbi.messaging.exchange.MessageExchangeWrapper;

import com.ebmwebsourcing.easycommons.lang.UncheckedException;
import com.ebmwebsourcing.easycommons.stream.InputStreamForker;
import com.ebmwebsourcing.easycommons.stream.ReaderInputStream;

/**
 * @author Roland Naudin - EBM WebSourcing
 */
public class SourcesForkerUtil {

    /**
     * A map of forked Source Streams to set the streams reusable
     */
    private static final Map<String, Map<String, InputStreamForker>> forkedStreams = new ConcurrentHashMap<String, Map<String, InputStreamForker>>();

    /**
     * Clean the potential duplicated Sources.
     * 
     * @param exchange
     * @throws IOException
     */
    public static final void cleanExchangeSources(final MessageExchange exchange)
            throws IOException {
        final Map<String, InputStreamForker> exchangeStreamForked = forkedStreams.remove(exchange
                .getExchangeId());
        if (exchangeStreamForked != null) {
            for (final InputStreamForker streamForker : exchangeStreamForked.values()) {
                streamForker.fork().close();
            }
        }
    }

    /**
     * Handle streaming issue. Duplicate sources if they are StreamSource. <br>
     * TODO : extend the mechanism to the attachments.
     * 
     * @param exchange
     * @throws MessagingException
     */
    public static final void forkExchangeSources(final MessageExchangeWrapper exchangeDecorator)
            throws MessagingException {

        Map<String, InputStreamForker> exchangeStreamForked = forkedStreams.get(exchangeDecorator
                .getExchangeId());
        if (exchangeStreamForked == null) {
            exchangeStreamForked = new HashMap<String, InputStreamForker>();
            forkedStreams.put(exchangeDecorator.getExchangeId(), exchangeStreamForked);
        }

        final Map<String, NormalizedMessage> messages = exchangeDecorator.getMessages();
        for (final Map.Entry<String, NormalizedMessage> messageEntry : messages.entrySet()) {
            // TODO Handle the Streaming issue for the Attachments??
            final String messageName = messageEntry.getKey();
            final Source content = messageEntry.getValue().getContent();
            if (content instanceof StreamSource) {
                InputStreamForker streamForker = exchangeStreamForked.get(messageName);
                if (streamForker != null) {
                    try {
                        streamForker = new InputStreamForker(streamForker.fork());
                    } catch (final IOException e) {
                        throw new UncheckedException(e);
                    }
                } else {
                    // As a StreamSource can be created from an InputStream or a Reader, we must manage both
                    final StreamSource streamContent = (StreamSource) content;
                    final InputStream isContent = streamContent.getInputStream();
                    if (isContent != null) {
                        // The StreamSource was created from an InputStream
                        streamForker = new InputStreamForker(isContent);
                    } else {
                        // The StreamSource was created from a Reader
                        // we wrap it as an InputStream
                        streamForker = new InputStreamForker(new ReaderInputStream(streamContent
                                .getReader()));
                    }
                }
                exchangeStreamForked.put(messageName, streamForker);
                Source source;
                try {
                    source = new StreamSource(streamForker.fork());
                } catch (final IOException e) {
                    throw new UncheckedException(e);
                }
                messageEntry.getValue().setContent(source);
            }
        }
    }

}
