/**
 * Copyright (c) 2005-2012 Fossil E-Commerce, 2005-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.system.classloader;

import java.io.IOException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.util.LoggingUtil;

/**
 * @since Petals 1.0
 * @author Rafael Marins - Fossil E-Commerce 
 * @author Adrien LOUIS - EBM WebSourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = ClassLoaderService.class))
public class ClassLoaderServiceImpl implements ClassLoaderService {

    /**
     * the common loader name
     */
    public static final String COMMON_LOADER = "common";

    /**
     * the shared library class loader.
     */
    private SharedLibrariesClassLoader sharedLibrariesClassLoader;

    /**
     * The class loaders table.
     */
    private Map<String, PetalsClassLoader> classLoaders = new HashMap<String, PetalsClassLoader>();

    /**
     * Parent ClassLoader for all JBI ClassLoaders.
     */
    private ClassLoader parentClassLoader;

    /**
     * Default constructor.
     */
    public ClassLoaderServiceImpl() {
        super();
    }

    LoggingUtil log;

    /**
     * The Configuration service fractal component
     */
    @Requires(name = "configuration")
    private ConfigurationService configurationService;

    // ----------------------------------------------------------------------
    // ClassLoadingService interface implementation
    // ----------------------------------------------------------------------

    /**
     * @see org.ow2.petals.system.classloader.ClassLoaderService#containsClassLoader(java.lang.String)
     */
    public boolean containsClassLoader(String classLoaderId) {
        this.log.call();

        return this.classLoaders.containsKey(classLoaderId);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.platform.classloader.ClassLoaderService#
     * createComponentClassLoader(java.lang.String, java.net.URL[],
     * java.util.List, boolean, java.util.List)
     */
    public URLClassLoader createComponentClassLoader(String classLoaderId, URL[] baseUrls,
            List<String> classpathUrls, boolean useParentFirst, List<String> sharedLibrariesNameList)
            throws PetalsException {
        this.log.start();

        PetalsClassLoader cl = null;
        try {
            cl = new ComponentClassLoader(baseUrls, classpathUrls, this.sharedLibrariesClassLoader,
                    sharedLibrariesNameList);
            cl.setParentFirst(useParentFirst);
        } catch (IOException e) {
            throw new PetalsException("Error while creating a PEtALS class loader", e);
        }

        this.classLoaders.put(classLoaderId, cl);

        this.log.end();
        return cl;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.platform.classloader.ClassLoaderService#
     * createSharedLibraryClassLoader(java.lang.String, java.net.URL[],
     * java.util.List, boolean)
     */
    public URLClassLoader createSharedLibraryClassLoader(String classLoaderId, URL[] baseUrls,
            List<String> classpathUrls, boolean useParentFirst) throws PetalsException {
        this.log.start();

        SharedLibraryClassLoader cl = null;
        try {
            cl = new SharedLibraryClassLoader(baseUrls, classpathUrls, this.parentClassLoader);
            cl.setParentFirst(useParentFirst);
        } catch (IOException e) {
            throw new PetalsException("Error while creating a class loader", e);
        }

        this.classLoaders.put(classLoaderId, cl);
        this.sharedLibrariesClassLoader.addSharedLibraryClassLoader(classLoaderId, cl);

        this.log.end();
        return cl;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.platform.classloader.ClassLoaderService#deleteClassLoader
     * (java.lang.String)
     */
    public void deleteClassLoader(String classLoaderId) {
        this.log.start();

        if (!this.containsClassLoader(classLoaderId)) {
            this.log.warning("No classloader with Id " + classLoaderId
                    + " loaded. Can not remove it");
            return;
        }

        final PetalsClassLoader classLoader = this.classLoaders.remove(classLoaderId);
        if (classLoader instanceof SharedLibraryClassLoader) {
            this.sharedLibrariesClassLoader.removeSharedLibraryClassLoader(classLoaderId);
        }
        classLoader.close();

        this.log.end();
    }

    // //////////////////////////////////////////////////////////////
    // clear class loader caches
    // ////////////////////////////////////////////////////////////////

    // TODO: rnaudin: remove this or is it usefull?

    // /**
    // * When uninstalling a component, some elements remains in the cache of
    // * certain classes, and the classLoader can not release the JARS
    // containing
    // * those elements
    // *
    // * referenced bug on Sun VM : 6232010
    // *
    // */
    // protected void cleanCache(PetalsClassLoader cLoader) {
    // this.log.call();
    //
    // this.cleanCache(cLoader, "java.io.ObjectInputStream", "subclassAudits");
    // this.cleanCache(cLoader, "java.io.ObjectOutputStream", "subclassAudits");
    // this.cleanCache(cLoader, "java.io.ObjectStreamClass", "localDescs");
    // this.cleanCache(cLoader, "java.io.ObjectStreamClass", "reflectors");
    // }
    //
    // /**
    // * Clear the cache contained in the specified class, for the specific
    // field
    // * (typically, streaming classes)
    // *
    // * referenced bug on Sun VM : 6232010
    // *
    // * @param the
    // * classLoader to clean (used when loading stream classes)
    // * @param className
    // * the name of the class containing the cache field
    // * @param fieldName
    // * the name of the cache field
    // */
    // @SuppressWarnings("unchecked")
    // protected void cleanCache(PetalsClassLoader cLoader, String className,
    // String fieldName) {
    //
    // Map cache = null;
    // try {
    // Class<?> clazz = cLoader.loadClass(className);
    // for (Field field : clazz.getDeclaredFields()) {
    // if (field.getType().equals(Map.class) ||
    // field.getType().equals(HashMap.class)) {
    // field.setAccessible(true);
    // cache = (Map) field.get(null);
    // if (cache != null) {
    // synchronized (cache) {
    // cache.clear();
    // }
    // }
    // }
    // }
    // } catch (Throwable ex) {
    // // class not loaded, nothing to do
    // }
    // }

    @LifeCycle(on = LifeCycleType.START)
    protected void start() {
        this.log = new LoggingUtil(java.util.logging.Logger.getLogger(Constants.FRACTAL_COMPONENT_LOGGER_NAME));
        this.log.start();

        boolean isIsolated = this.configurationService.getContainerConfiguration()
                .isIsolateJBIClassLoaders();

        if (isIsolated) {
            this.parentClassLoader = ClassLoader.getSystemClassLoader();

        } else {
            this.parentClassLoader = ClassLoaderServiceImpl.class.getClassLoader();
        }
        this.classLoaders = new HashMap<String, PetalsClassLoader>();
        this.sharedLibrariesClassLoader = new SharedLibrariesClassLoader(this.parentClassLoader);

        this.log.end();
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() {
        this.log.start();

        for (String classLoaderId : this.classLoaders.keySet()) {
            this.sharedLibrariesClassLoader.removeSharedLibraryClassLoader(classLoaderId);
            this.classLoaders.get(classLoaderId).close();
        }

        this.classLoaders = null;
        this.sharedLibrariesClassLoader = null;
        this.parentClassLoader = null;

        this.log.end();
    }
}
