/**
 * Copyright (c) 2005-2012 Fossil E-Commerce, 2005-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.system.classloader;

import java.io.InputStream;
import java.net.URL;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * This classLoader englobes all SharedLibraries classLoader. 
 * <p>
 * When the ClassLoader of a component does not found a class, This classLoader search in
 * all the SLClassLoader the class. If the class is not found in those
 * SLClassLoader, search in the parent classLoader.
 * </p>
 * 
 * @since Petals 1.0
 * @author Adrien LOUIS - EBM WebSourcing
 * @author ddesjardins - EBM WebSourcing
 */
public class SharedLibrariesClassLoader extends ClassLoader {

    protected Map<String, SharedLibraryClassLoader> slClassLoaders;

    /**
     * Create a SharedLibrariesClassLoader, with SystemClassLoader as parent.
     * 
     */
    public SharedLibrariesClassLoader(ClassLoader parent) {
        super(parent);

        this.slClassLoaders = new ConcurrentHashMap<String, SharedLibraryClassLoader>();
    }

    /**
     * add a shared library class loader.
     * 
     * @param id
     *            the name of classloader
     * @param cl
     *            the class loader
     */
    public void addSharedLibraryClassLoader(final String id, final SharedLibraryClassLoader cl) {
        this.slClassLoaders.put(id, cl);
    }

    /**
     * remove a shared library class loader.
     * 
     * @param id
     *            the name of class loader
     */
    public void removeSharedLibraryClassLoader(final String id) {
        this.slClassLoaders.remove(id);

    }

    // ----------------------------------------------------------------------
    // overriding methods
    // ----------------------------------------------------------------------

    /**
     * Ask to each referenced SLClassLoader (which name is specified in the
     * list) to find the class. If none of the SLclassLoader knows this class,
     * throw an exception
     * 
     * @param className
     *            the name of the class
     * @throws ClassNotFoundException
     * @see java.net.URLClassLoader#findClass(java.lang.String)
     */
    /*
     * protected Class<?> findClass(String className, List<String>
     * sharedLibrariesNameList) throws ClassNotFoundException { Class<?> result =
     * null; if (sharedLibrariesNameList != null) { synchronized
     * (sharedLibrariesNameList) { for (Iterator<String> iter =
     * sharedLibrariesNameList.iterator(); iter.hasNext() && result == null;) {
     * SharedLibraryClassLoader element = slClassLoaders.get(iter.next()); if
     * (element != null) { try { result = element.findClass(className); } catch
     * (ClassNotFoundException e) { // the class is not found with this class
     * loader, // search in the next one } } } } } if (result == null) { throw
     * new ClassNotFoundException(); } return result; }
     */

    /**
     * Ask to each referenced SLClassLoader (which name is specified in the
     * list) to find a loaded class. If none of the SLclassLoader knows this
     * class, throw an exception. This method call the "findLoadedClazz" method
     * of the SLclassloaders.
     * 
     * @throws ClassNotFoundException
     * @see java.net.URLClassLoader#findClass(java.lang.String)
     */
    /*
     * protected Class<?> findLoadedClass(String className, List<String>
     * sharedLibrariesNameList) { Class<?> clazz = null; // search in the
     * sharedlibraries class loaders if (sharedLibrariesNameList != null) {
     * synchronized (sharedLibrariesNameList) { for (Iterator<String> iter =
     * sharedLibrariesNameList.iterator(); iter.hasNext() && clazz == null;) {
     * SharedLibraryClassLoader element = slClassLoaders.get(iter.next());
     * 
     * if (element != null) { clazz = element.findLoadedClazz(className); } } } }
     * return clazz; }
     */

    /**
     * Search in all referenced SharedLibrariesClassLoaders for the requested
     * resource.
     * 
     */
    protected URL getResource(String resourceName, List<String> sharedLibrariesNameList) {
        URL url = null;
        if (sharedLibrariesNameList != null) {
            synchronized (sharedLibrariesNameList) {
                for (Iterator<String> iter = sharedLibrariesNameList.iterator(); iter.hasNext()
                        && url == null;) {
                    SharedLibraryClassLoader element = slClassLoaders.get(iter.next());
                    if (element != null) {
                        url = element.getResource(resourceName);
                    }
                }
            }
        }
        return url;
    }

    /**
     * Search in all referenced SharedLibrariesClassLoaders for the requested
     * resource.
     * 
     */
    protected InputStream getResourceAsStream(String resourceName,
            List<String> sharedLibrariesNameList) {
        InputStream inputStream = null;
        if (sharedLibrariesNameList != null) {
            synchronized (sharedLibrariesNameList) {
                for (Iterator<String> iter = sharedLibrariesNameList.iterator(); iter.hasNext()
                        && inputStream == null;) {
                    SharedLibraryClassLoader element = slClassLoaders.get(iter.next());
                    if (element != null) {
                        inputStream = element.getResourceAsStream(resourceName);
                    }
                }
            }
        }
        return inputStream;
    }

    /**
     * Search in all referenced SharedLibrariesClassLoaders for the requested
     * class. If the class is not found, search in the parent classLoader.
     * 
     * @see java.lang.ClassLoader#loadClass(java.lang.String, boolean)
     */
    protected synchronized Class<?> loadClass(String className, boolean resolve,
            List<String> sharedLibrariesNameList) throws ClassNotFoundException {
        Class<?> clazz = null;
        if (sharedLibrariesNameList != null && !sharedLibrariesNameList.isEmpty()) {
            for (Iterator<String> iter = sharedLibrariesNameList.iterator(); iter.hasNext()
                    && clazz == null;) {
                SharedLibraryClassLoader element = slClassLoaders.get(iter.next());
                if (element != null) {
                    try {
                        clazz = element.loadClass(className, resolve);
                    } catch (ClassNotFoundException e) {
                        // Nothing to do because we have to search in all
                        // SLClassloaders
                    }
                }
            }
            if (clazz == null) {
                throw new ClassNotFoundException(className);
            }
        } else {
            // No shared libraries to search, delegate classloading to parent
            clazz = super.loadClass(className);
        }
        return clazz;
    }

    /**
     * @return Returns a String representation of the
     *         SharedLibrairiesClassLoader
     */
    @Override
    public String toString() {
        StringBuffer sb = new StringBuffer();

        sb.append("classloader : " + getClass().getName() + "\n");

        synchronized (slClassLoaders) {
            sb.append("\t " + slClassLoaders.values());
        }

        return sb.toString();
    }
}
