/**
 * Copyright (c) 2006-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.system.systemstate;

import java.io.File;
import java.io.IOException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import javax.jbi.management.LifeCycleMBean;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.ow2.petals.microkernel.api.container.Installer;
import org.ow2.petals.microkernel.api.system.SystemStateService;
import org.ow2.petals.microkernel.api.system.repository.RepositoryService;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.system.systemstate.exception.ComponentAlreadyExistsException;
import org.ow2.petals.microkernel.system.systemstate.exception.ServiceAssemblyAlreadyExistsException;
import org.ow2.petals.microkernel.system.systemstate.exception.SharedLibraryAlreadyExistsException;
import org.ow2.petals.systemstate.SystemStateBuilder;
import org.ow2.petals.systemstate.SystemStateException;
import org.ow2.petals.systemstate.generated.Component;
import org.ow2.petals.systemstate.generated.ObjectFactory;
import org.ow2.petals.systemstate.generated.ServiceAssembly;
import org.ow2.petals.systemstate.generated.SharedLibrary;
import org.ow2.petals.systemstate.generated.SystemState;

/**
 * Creates and reads state files for components, shared libs and service assemblies. 
 * <p>
 * State files are text files that keep the state of JBI entities to
 * allow state retreval after Petals system shutdown or crash.
 * </p>
 * 
 * @author Adrien Louis - EBM WebSourcing
 * @author Roland Naudin - EBM WebSourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = SystemStateService.class))
public class SystemStateServiceImpl implements SystemStateService {

    private static final String SYSTEMSTATE_FILE = "system-state.xml";

    /**
     * logger wrapper
     */
    private LoggingUtil log;

    private ObjectFactory objectFactory;

    @Requires(name = "repository")
    private RepositoryService repositoryService;

    private SystemState systemState;

    private File systemStateFile;

    public SystemStateServiceImpl() {
        super();
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.systemstate.SystemStateService#
     * createComponentStateHolder(java.lang.String, java.net.URL,
     * java.lang.String)
     */
    public Component createComponentStateHolder(String componentName, URL installUrl,
            String archiveName) throws ComponentAlreadyExistsException, SystemStateException {

        final Component componentToAdd = this.objectFactory.createComponent();
        componentToAdd.setName(componentName);
        componentToAdd.setInstallURL(installUrl.toExternalForm());
        componentToAdd.setInstallState(Installer.UNINSTALLED);
        componentToAdd.setArchiveName(archiveName);
        componentToAdd.setLifecycleState(LifeCycleMBean.UNKNOWN);

        synchronized (this.systemState) {
            final List<Component> components = this.systemState.getComponents().getComponent();

            // We check that the component does not already exist
            for (final Component component : components) {
                if (component.getName().equals(componentName)) {
                    throw new ComponentAlreadyExistsException(componentName);
                }
            }

            components.add(componentToAdd);
            SystemStateBuilder.buildXmlFileSystemState(this.systemState, this.systemStateFile);
        }

        return componentToAdd;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.systemstate.SystemStateService#
     * createServiceAssemblyStateHolder(java.lang.String, java.lang.String)
     */
    public ServiceAssembly createServiceAssemblyStateHolder(String saName, URL installUrl,
            String archiveName) throws ServiceAssemblyAlreadyExistsException, SystemStateException {

        final ServiceAssembly serviceAssemblyToAdd = this.objectFactory.createServiceAssembly();
        serviceAssemblyToAdd.setName(saName);
        serviceAssemblyToAdd.setInstallURL(installUrl.toExternalForm());
        serviceAssemblyToAdd.setArchiveName(archiveName);
        serviceAssemblyToAdd.setLifecycleState(LifeCycleMBean.UNKNOWN);

        synchronized (this.systemState) {
            final List<ServiceAssembly> serviceAssemblies = this.systemState.getServiceAssemblies()
                    .getServiceAssembly();

            // We check that the service assembly does not already exist
            for (final ServiceAssembly serviceAssembly : serviceAssemblies) {
                if (serviceAssembly.getName().equals(saName)) {
                    throw new ServiceAssemblyAlreadyExistsException(saName);
                }
            }

            serviceAssemblies.add(serviceAssemblyToAdd);
            SystemStateBuilder.buildXmlFileSystemState(this.systemState, this.systemStateFile);
        }

        return serviceAssemblyToAdd;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.systemstate.SystemStateService#
     * createSharedLibraryStateHolder(java.lang.String, java.lang.String,
     * java.lang.String)
     */
    public SharedLibrary createSharedLibraryStateHolder(String slName, String version,
            URL installUrl, String archiveName) throws SystemStateException {
        final SharedLibrary sharedLibraryToAdd = this.objectFactory.createSharedLibrary();
        sharedLibraryToAdd.setName(slName);
        sharedLibraryToAdd.setVersion(version);
        sharedLibraryToAdd.setInstallURL(installUrl.toExternalForm());
        sharedLibraryToAdd.setArchiveName(archiveName);

        synchronized (this.systemState) {
            final List<SharedLibrary> sharedLibraries = this.systemState.getSharedLibraries()
                    .getSharedLibrary();

            // We check that the service assembly does not already exist
            for (final SharedLibrary sharedLibrary : sharedLibraries) {
                if (sharedLibrary.getName().equals(slName)
                        && sharedLibrary.getVersion().equals(version)) {
                    throw new SharedLibraryAlreadyExistsException(slName, version);
                }
            }

            sharedLibraries.add(sharedLibraryToAdd);
            SystemStateBuilder.buildXmlFileSystemState(this.systemState, this.systemStateFile);
        }

        return sharedLibraryToAdd;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.systemstate.SystemStateService#
     * deleteComponentStateHolder(java.lang.String)
     */
    public Component deleteComponentStateHolder(String componentName) throws SystemStateException {
        Component componentStateHolder = null;
        for (final Component _componentStateHolder : this.systemState.getComponents()
                .getComponent()) {
            if (componentName.equals(_componentStateHolder.getName())) {
                componentStateHolder = _componentStateHolder;
                break;
            }
        }

        if (componentStateHolder != null) {
            synchronized (this.systemState) {
                this.systemState.getComponents().getComponent().remove(componentStateHolder);
                SystemStateBuilder.buildXmlFileSystemState(this.systemState, this.systemStateFile);
            }
        }

        return componentStateHolder;

    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.systemstate.SystemStateService#
     * deleteServiceAssemblyStateHolder(java.lang.String)
     */
    public ServiceAssembly deleteServiceAssemblyStateHolder(String saName)
            throws SystemStateException {
        ServiceAssembly serviceAssemblyStateHolder = null;
        for (final ServiceAssembly _serviceAssemblyStateHolder : this.systemState
                .getServiceAssemblies().getServiceAssembly()) {
            if (saName.equals(_serviceAssemblyStateHolder.getName())) {
                serviceAssemblyStateHolder = _serviceAssemblyStateHolder;
                break;
            }
        }

        if (serviceAssemblyStateHolder != null) {
            synchronized (this.systemState) {
                this.systemState.getServiceAssemblies().getServiceAssembly().remove(
                        serviceAssemblyStateHolder);
                SystemStateBuilder.buildXmlFileSystemState(this.systemState, this.systemStateFile);
            }
        }

        return serviceAssemblyStateHolder;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.systemstate.SystemStateService#
     * deleteSharedLibraryStateHolder(java.lang.String, java.lang.String)
     */
    public SharedLibrary deleteSharedLibraryStateHolder(String slName, String version)
            throws SystemStateException {
        SharedLibrary sharedLibraryStateHolder = null;
        for (final SharedLibrary _sharedLibraryStateHolder : this.systemState.getSharedLibraries()
                .getSharedLibrary()) {
            if (slName.equals(_sharedLibraryStateHolder.getName())
                    && version.equals(_sharedLibraryStateHolder.getVersion())) {
                sharedLibraryStateHolder = _sharedLibraryStateHolder;
                break;
            }
        }

        if (sharedLibraryStateHolder != null) {
            synchronized (this.systemState) {
                this.systemState.getSharedLibraries().getSharedLibrary().remove(
                        sharedLibraryStateHolder);
                SystemStateBuilder.buildXmlFileSystemState(this.systemState, this.systemStateFile);
            }
        }

        return sharedLibraryStateHolder;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.systemstate.SystemStateService#getComponentStateHolder
     * (java.lang.String)
     */
    public Component getComponentStateHolder(String componentName) {
        for (final Component componentStateHolder : this.systemState.getComponents().getComponent()) {
            if (componentName.equals(componentStateHolder.getName())) {
                return componentStateHolder;
            }
        }

        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.systemstate.SystemStateService#getComponentStateHolders
     * ()
     */
    public List<Component> getComponentStateHolders() {
        return new ArrayList<Component>(this.systemState.getComponents().getComponent());
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.systemstate.SystemStateService#
     * getServiceAssemblyStateHolder(java.lang.String)
     */
    public ServiceAssembly getServiceAssemblyStateHolder(String saName) {
        synchronized (this.systemState) {
            for (final ServiceAssembly serviceAssemblyStateHolder : this.systemState
                    .getServiceAssemblies().getServiceAssembly()) {
                if (saName.equals(serviceAssemblyStateHolder.getName())) {
                    return serviceAssemblyStateHolder;
                }
            }
        }

        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.systemstate.SystemStateService#
     * getServiceAssemblyStateHolders()
     */
    public List<ServiceAssembly> getServiceAssemblyStateHolders() {
        return new ArrayList<ServiceAssembly>(this.systemState.getServiceAssemblies()
                .getServiceAssembly());
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.systemstate.SystemStateService#
     * getSharedLibraryStateHolder(java.lang.String, java.lang.String)
     */
    public SharedLibrary getSharedLibraryStateHolder(String slName, String version) {
        synchronized (this.systemState) {
            for (final SharedLibrary sharedLibraryStateHolder : this.systemState
                    .getSharedLibraries().getSharedLibrary()) {
                if (slName.equals(sharedLibraryStateHolder.getName())
                        && version.equals(sharedLibraryStateHolder.getVersion())) {
                    return sharedLibraryStateHolder;
                }
            }
        }

        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.systemstate.SystemStateService#
     * getSharedLibraryStateHolders()
     */
    public List<SharedLibrary> getSharedLibraryStateHolders() {
        return new ArrayList<SharedLibrary>(this.systemState.getSharedLibraries()
                .getSharedLibrary());
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.systemstate.SystemStateService#
     * getSharedLibraryStateHolders(java.lang.String)
     */
    public List<SharedLibrary> getSharedLibraryStateHolders(String slName) {
        final List<SharedLibrary> sharedLibrarySharedHolders = new ArrayList<SharedLibrary>();
        synchronized (this.systemState) {
            for (final SharedLibrary sharedLibraryStateHolder : this.systemState
                    .getSharedLibraries().getSharedLibrary()) {
                if (slName.equals(sharedLibraryStateHolder.getName())) {
                    sharedLibrarySharedHolders.add(sharedLibraryStateHolder);
                }
            }
        }

        return sharedLibrarySharedHolders;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.systemstate.SystemStateService#isComponentLoaded
     * (java.lang.String)
     */
    public boolean isComponentInstalled(String componentName) {
        boolean result = false;

        final Component componentStateHolder = this.getComponentStateHolder(componentName);
        if (Installer.INSTALLED.equals(componentStateHolder.getInstallState())) {
            result = true;
        }

        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.systemstate.SystemStateService#
     * updateComponentInstallationState(java.lang.String, java.lang.String)
     */
    public void updateComponentInstallationState(String componentName, String installState)
            throws SystemStateException {
        synchronized (this.systemState) {
            for (final Component componentStateHolder : this.systemState.getComponents()
                    .getComponent()) {
                if (componentName.equals(componentStateHolder.getName())) {
                    componentStateHolder.setInstallState(installState);
                    SystemStateBuilder.buildXmlFileSystemState(this.systemState,
                            this.systemStateFile);
                    break;
                }
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.systemstate.SystemStateService#
     * updateComponentLifeCycleState(java.lang.String, java.lang.String)
     */
    public void updateComponentLifeCycleState(String componentName, String lifecycleState)
            throws SystemStateException {
        synchronized (this.systemState) {
            for (final Component componentStateHolder : this.systemState.getComponents()
                    .getComponent()) {
                if (componentName.equals(componentStateHolder.getName())) {
                    componentStateHolder.setLifecycleState(lifecycleState);
                    SystemStateBuilder.buildXmlFileSystemState(this.systemState,
                            this.systemStateFile);
                    break;
                }
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.systemstate.SystemStateService#
     * updateServiceAssemblyState(java.lang.String, java.lang.String)
     */
    public void updateServiceAssemblyState(String saName, String lifecycleState)
            throws SystemStateException {
        synchronized (this.systemState) {
            for (final ServiceAssembly serviceAssemblyStateHolder : this.systemState
                    .getServiceAssemblies().getServiceAssembly()) {
                if (saName.equals(serviceAssemblyStateHolder.getName())) {
                    serviceAssemblyStateHolder.setLifecycleState(lifecycleState);
                    SystemStateBuilder.buildXmlFileSystemState(this.systemState,
                            this.systemStateFile);
                    break;
                }
            }
        }
    }

    /**
     * Initialize the SystemState Service.
     * 
     * @throws IOException
     * @throws SystemStateException
     * 
     * @throws URISyntaxException
     */
    @LifeCycle(on = LifeCycleType.START)
    protected void start() throws IOException, SystemStateException {
        this.log = new LoggingUtil(
                java.util.logging.Logger.getLogger(SystemStateService.COMPONENT_LOGGER_NAME));
        this.log.start();

        this.objectFactory = new ObjectFactory();
        final File repoPath = this.repositoryService.getRepositoryDirectory();

        if (!repoPath.exists()) {
            // TODO: Manage error
            repoPath.mkdirs();
        }

        try {
            this.systemStateFile = new File(repoPath.getAbsolutePath(), SYSTEMSTATE_FILE);
            if (!this.systemStateFile.exists()) {
                this.systemStateFile.createNewFile();
                this.systemState = this.objectFactory.createSystemState();
                this.systemState.setComponents(this.objectFactory.createComponents());
                this.systemState.setServiceAssemblies(this.objectFactory.createServiceAssemblies());
                this.systemState.setSharedLibraries(this.objectFactory.createSharedLibraries());
                SystemStateBuilder.buildXmlFileSystemState(this.systemState, this.systemStateFile);
            } else {
                this.systemState = SystemStateBuilder.createSystemState(this.systemStateFile);
            }
        } catch (SystemStateException e) {
            this.log.error("Failed to start System State Service", e);
            throw e;
        } catch (IOException e) {
            this.log.error("Failed to start System State Service", e);
            throw e;
        }

        this.log.end();
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() {
        this.log.call();
    }
}
