/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.transport.local;

import java.util.ConcurrentModificationException;
import java.util.List;
import java.util.Vector;

import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeWrapper;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.transport.TransportException;
import org.ow2.petals.microkernel.transport.TransportListener;
import org.ow2.petals.microkernel.transport.Transporter;
import org.ow2.petals.microkernel.transport.local.monitoring.LocalTransporterMonitoringItf;
import org.ow2.petals.microkernel.transport.util.TransportSendContext;
import org.ow2.petals.microkernel.transport.util.TransporterUtil;

/**
 * @author Roland NAUDIN - EBM WebSourcing
 */
@FractalComponent
@Provides(interfaces = { @Interface(name = "service", signature = Transporter.class) })
public class LocalTransporterImpl implements Transporter {

    /**
     * the Transporter listener (router) Fractal component
     */
    @Requires(name = "transportlistener")
    private TransportListener transportListener;

    /**
     * The local transporter monitoring service - Fractal component
     */
    @Requires(name = "localtransportermonitoring")
    private LocalTransporterMonitoringItf localTransporterMonitoring;

    /**
     * The logger.
     */
    private LoggingUtil log;

    /**
     * List of exchanges that are blocked during a synchronous send
     */
    private List<MessageExchange> pendingSyncExchanges;

    /**
     * Traffic flag to handle stop of traffic
     */
    private boolean stopTraffic;

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.transport.Transporter#prepareStop()
     */
    public void stopTraffic() {
        this.log.start();
        this.stopTraffic = true;

        // Awake all the pending 'SYNC_SEND' issuer threads to abort the
        // invocation
        boolean redo = true;
        while (redo) {
            try {
                for (MessageExchange messageExchangeImpl : this.pendingSyncExchanges) {
                    synchronized (messageExchangeImpl) {
                        messageExchangeImpl.notify();
                    }
                }
                redo = false;
            } catch (ConcurrentModificationException e) {
                // Try again
            }
        }

        this.log.end();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.transport.Transporter#send(org.ow2.petals.jbi.messaging
     * .exchange.MessageExchangeDecorator,
     * org.ow2.petals.transport.util.TransportSendContext)
     */
    public void send(final MessageExchangeWrapper exchangeDecorator,
            final TransportSendContext transportContext) throws TransportException {
        this.log.start();

        this.checkTransporterState();

        this.localTransporterMonitoring.incMessageSentProbe(exchangeDecorator);

        // notify that the exchange is effectively sent
        this.transportListener.exchangeSent(exchangeDecorator);

        if (TransporterUtil.getSyncMode(exchangeDecorator, true)) {
            this.pendingSyncExchanges.remove(exchangeDecorator);
            synchronized (exchangeDecorator) {
                exchangeDecorator.notify();
            }
        } else {
            this.transportListener.onExchange(exchangeDecorator);
        }

        this.log.end();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.transport.Transporter#sendSync(org.ow2.petals.jbi.messaging
     * .exchange.MessageExchangeDecorator,
     * org.ow2.petals.transport.util.TransportSendContext)
     */
    public void sendSync(final MessageExchangeWrapper exchangeDecorator,
            final TransportSendContext transportContext) throws TransportException {
        this.log.start();

        this.checkTransporterState();

        // set the relevant properties to mark the message as a
        // synchronous send
        boolean isOverSync;
        try {
            isOverSync = TransporterUtil.setSendSyncProperties(exchangeDecorator);
        } catch (MessagingException e) {
            throw new TransportException(e);
        }

        this.localTransporterMonitoring.incMessageSentProbe(exchangeDecorator);

        // notify that the exchange is effectively sent
        this.transportListener.exchangeSent(exchangeDecorator);

        synchronized (exchangeDecorator) {
            if (isOverSync) {
                // in this case the current send sync is a response of a
                // previous send sync
                exchangeDecorator.notify();
            } else {
                this.pendingSyncExchanges.add(exchangeDecorator);
            }

            try {
                if (!isOverSync) {
                    this.transportListener.onExchange(exchangeDecorator);
                }

                // wait for a response or a timeout reached
                exchangeDecorator.wait(transportContext.timeout);
            } catch (InterruptedException e) {
                throw new TransportException(
                        "Failed to receive the response message of a synchronous send", e);
            }
        }

        // reset the pending exchange it is an 'overSync' exchange
        if (!TransporterUtil.updateSyncProperties(exchangeDecorator)
                && this.pendingSyncExchanges.contains(exchangeDecorator)) {
            // If exchange is in the pending list,
            // it means that the timeout has been reached or the transporter is
            // stopping
            this.checkTransporterState();

            // try to remove the exchange if a timeout
            this.pendingSyncExchanges.remove(exchangeDecorator);
            exchangeDecorator.setTimeout(true);

            this.log.warning("Failed to send synchronously the exchange: "
                    + exchangeDecorator.getExchangeId() + ". Timeout occured");

        }

        this.log.end();
    }

    @LifeCycle(on = LifeCycleType.START)
    protected void start() throws Exception {
        this.log = new LoggingUtil(java.util.logging.Logger.getLogger(Constants.FRACTAL_COMPONENT_LOGGER_NAME));
        this.log.start();

        // TODO: This queue should be sizable
        this.pendingSyncExchanges = new Vector<MessageExchange>(100);

        this.log.end();
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() throws Exception {
        this.log.call();
    }

    /**
     * Check if the transporter is running, if not, raise an exception.
     * 
     * @throws TransportException
     */
    private void checkTransporterState() throws TransportException {
        if (this.stopTraffic) {
            Thread.currentThread().interrupt();
            throw new TransportException("The Transporter traffic is stopped",
                    new InterruptedException());
        }
    }

}
