/**
 * Copyright (c) 2009-2012 Capgemini, 2009-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.transport.platform.nio.selector;

import java.io.IOException;
import java.net.Socket;
import java.nio.channels.ClosedChannelException;
import java.nio.channels.ClosedSelectorException;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.nio.channels.spi.AbstractSelectableChannel;
import java.util.Iterator;
import java.util.Queue;
import java.util.concurrent.LinkedBlockingQueue;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.ow2.petals.microkernel.api.util.LoggingUtil;

/**
 * The NIO selector.
 * @author Christophe DENEUX - Capgemini Sud
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = NioSelector.class))
public class NioSelectorAgent implements Runnable, NioSelector {

    /**
     * Inner class.
     * 
     * @author cdeneux
     * 
     */
    private class PendingRegistration {
        protected AbstractSelectableChannel socketChannel;

        protected int key;

        protected Object attachment;

        public PendingRegistration(final AbstractSelectableChannel socketChannel, final int key,
                final Object attachment) {
            this.socketChannel = socketChannel;
            this.key = key;
            this.attachment = attachment;
        }
    }

    private LoggingUtil log;

    /**
     * The socket selector.
     */
    private Selector selector;

    /**
     * The selector agent thread as a singleton to be used by the server agent
     * and the client agent.
     */
    private Thread selectorAgentThread;

    /**
     * Channel registration queue
     */
    private final Queue<PendingRegistration> registrationQueue = new LinkedBlockingQueue<PendingRegistration>();

    public final void register(final int key, final AbstractSelectableChannel channel,
            final Object attachement) throws ClosedChannelException {

        this.registrationQueue.offer(new PendingRegistration(channel, key, attachement));

        this.selector.wakeup();
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Runnable#run()
     */
    public final void run() {
        boolean toContinue = true;
        while (toContinue) {

            PendingRegistration pendingRegistration = null;
            // Process pending registrations
            while (!this.registrationQueue.isEmpty()) {
                try {
                    pendingRegistration = this.registrationQueue.poll();

                    pendingRegistration.socketChannel.register(this.selector,
                            pendingRegistration.key, pendingRegistration.attachment);
                } catch (final ClosedChannelException e) {
                    final String socketName;
                    if (pendingRegistration.socketChannel instanceof SocketChannel) {
                        // client Channel
                        socketName = ((SocketChannel) (pendingRegistration.socketChannel)).socket()
                                .toString();
                    } else {
                        // server Channel
                        socketName = ((ServerSocketChannel) (pendingRegistration.socketChannel))
                                .socket().toString();
                    }
                    this.log.error("Socket " + socketName + " : " + e.getMessage(), e);
                }
            }

            try {
                // Wait for an event on the registered channels
                if (this.selector.select() > 0) {

                    // Iterate over the set of keys for which events are
                    // available
                    final Iterator<SelectionKey> selectedKeys = this.selector.selectedKeys()
                            .iterator();
                    while (selectedKeys.hasNext()) {
                        final SelectionKey key = selectedKeys.next();
                        selectedKeys.remove();

                        if (key.isValid()) {
                            // Check what event is available and deal with it
                            if (key.isAcceptable()) {
                                this.accept(key);
                            } else if (key.isReadable()) {
                                this.read(key);
                            } else if (key.isWritable()) {
                                this.write(key);
                            } else if (key.isConnectable()) {
                                this.endsConnection(key);
                            } else {
                                // All events should be taken into account in
                                // previous tests
                                this.log.warning("Unexpected event");
                            }
                        }
                    }
                }
            } catch (final ClosedSelectorException e) {
                // The selector is closed, we stop the loop
                toContinue = false;
            } catch (final IOException e) {
                this.log.error(e.getMessage(), e);
            }
        }
    }

    @LifeCycle(on = LifeCycleType.START)
    protected void start() throws IOException {
        this.log = new LoggingUtil(java.util.logging.Logger.getLogger(Constants.FRACTAL_COMPONENT_LOGGER_NAME));
        this.log.start();

        this.selectorAgentThread = new Thread(this, "SelectorAgent Thread");
        this.selectorAgentThread.setDaemon(true);

        this.selector = Selector.open();
        this.selectorAgentThread.start();

        this.log.end();
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() throws Exception {
        this.log.start();

        this.selector.close();
        this.selectorAgentThread.interrupt();

        this.log.end();
    }

    /**
     * Accept a connection from a client.
     * 
     * @param key
     * @throws IOException
     */
    private final void accept(final SelectionKey key) throws IOException {
        this.log.start();

        // For an accept to be pending the channel must be a server socket
        // channel.
        final ServerSocketChannel serverSocketChannel = (ServerSocketChannel) key.channel();

        if (serverSocketChannel.isOpen()) {
            // Accept the connection and make it non-blocking
            final SocketChannel socketChannel = serverSocketChannel.accept();
            if (socketChannel != null) {
                final Socket socket = socketChannel.socket();
                socket.setKeepAlive(true);
                // TODO: Algo de naggle
                // TODO: taille des paquets TCP en accord avec ceux de IP sur le WAN
                socketChannel.configureBlocking(false);

                // Register the new socket channel with its associated context into
                // the selector, indicating we'd like to be notified when there's
                // data waiting to be read.
                final Object callback = key.attachment();
                if (callback instanceof NioServer) {
                    ((NioServer) callback).onConnectionIsAccepted(socketChannel, this.selector);
                } else {
                    // This problem should not occur because it's a bug
                    throw new IOException("Unexpected key attachment on OP_ACCEPT: "
                            + callback.getClass().getName());
                }
            } else {
                // In non-blocking mode, accept should always returns a not null
                // value, except in case of the connection process is not correctly
                // implemented.
                throw new IOException("Accepted socket but not already created.");
            }
        }

        this.log.end();
    }

    /**
     * Ends a client connection
     * 
     * @param key
     * @throws IOException
     */
    private final void endsConnection(final SelectionKey key) throws IOException {

        this.log.start();

        try {
            final Object attachment = key.attachment();
            NioClient nioClient;
            if (attachment instanceof NioClient) {
                nioClient = (NioClient) attachment;
            } else {
                // This problem should not occur because it's a bug
                throw new IOException("Unexpected key attachment on OP_CONNECT: "
                        + attachment.getClass().getName());
            }

            try {
                final SocketChannel socketChannel = (SocketChannel) key.channel();

                // Finish the connection.
                if (socketChannel.finishConnect()) {
                    // We report on the client side the connection establishment
                    nioClient
                            .onIsConnected(new OutputSocketChannelContext(socketChannel, nioClient));
                } else {
                    // In non-blocking mode, finishConnect should always returns
                    // true, except in case of the connection process is not
                    // correct on the client side (connect, next register to
                    // OP_CONNECT).
                    throw new IOException("Unable to finish the socket connection");
                }
            } catch (final IOException e) {
                this.log.error("Unable to connect: " + e.getMessage());
                nioClient.onConnectionError(e);
            }
        } finally {
            this.log.end();
        }
    }

    private void read(final SelectionKey key) throws IOException {
        this.log.start();

        final Object attach = key.attachment();
        if (attach instanceof InputSocketChannelContext) {
            ((InputSocketChannelContext) attach).read(key);
        } else if (attach instanceof OutputSocketChannelContext) {
            ((OutputSocketChannelContext) attach).onClose(key);
        } else {
            // This problem should not occur because it's a bug
            throw new IOException("Unexpected key attachment on OP_READ: "
                    + attach.getClass().getName());
        }

        this.log.end();
    }

    private void write(final SelectionKey key) {
        this.log.start();

        final Object attach = key.attachment();
        if (attach instanceof OutputSocketChannelContext) {
            ((OutputSocketChannelContext) attach).write(key);
        } else {
            // This problem should not occur because it's a bug
            this.log.error("Unexpected key attachment on OP_WRITE: " + attach.getClass().getName());
        }

        this.log.end();
    }

}
