/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.util;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.fail;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.zip.ZipFile;

import org.apache.commons.io.FileUtils;
import org.junit.Before;
import org.junit.Test;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.microkernel.api.util.ZipUtil;

/**
 * Class used to test the zip util.
 * @author ddesjardins - EBM WebSourcing
 */
public class ZipUtilTest {

    /**
     * Base directory of the project
     */
    private File baseDir = null;

    /**
     * Test resources directory of the project
     */
    private File testResourcesDir = null;

    /**
     * Directory where extracted file will be placed
     */
    private File extractedDir = null;

    /**
     * Set up the base directory for the test
     */
    @Before
    public void setUp() {
        String baseDirPath = this.getClass().getResource(".").getPath();
        baseDirPath = baseDirPath.substring(0, baseDirPath.indexOf("target"));
        baseDirPath = baseDirPath.substring(baseDirPath.indexOf(":") + 1);
        try {
            baseDir = new File(new URI("file:" + baseDirPath).normalize());
        } catch (URISyntaxException e) {
            e.printStackTrace();
        }

        testResourcesDir = new File(baseDir, "src" + File.separator + "test" + File.separator
                + "resources");

        extractedDir = new File(testResourcesDir, "extracted");
    }

    /**
     * Try to extract a zip into a directory
     * 
     * @throws PetalsException
     * @throws IOException
     */
    @Test
    public void testExplodeZip() throws PetalsException, IOException {
        extractedDir.delete();
        extractedDir.mkdir();
        ZipUtil.unzipArchive(new File(testResourcesDir, "test-ziputil.zip"), extractedDir);
        if (extractedDir.listFiles().length != 3) {
            fail();
        }
        FileUtils.forceDelete(extractedDir);
    }

    /**
     * Test exception throwing
     * 
     * @throws PetalsException
     * @throws IOException
     */
    @Test
    public void testExplodeZip1() {
        try {
            ZipUtil.unzipArchive(null, extractedDir);
            fail();
        } catch (NullPointerException e) {
            // NOP, expected result
        } catch (IOException e) {
            fail("IOException raised: " + e.getMessage());
        }
    }

    /**
     * Try to get an entry of a zip as a temporary file
     * 
     * @throws IOException
     * @throws PetalsException
     */
    @Test
    public void testGetEntryAsTemp() throws IOException, PetalsException {
        assertNotNull(ZipUtil.getEntryAsTemp(new ZipFile(testResourcesDir.getAbsolutePath()
                + File.separator + "test-ziputil.zip"), "META-INF/jbi.xml"));
    }

    /**
     * Try to get an entry that does not exist in zip as a temporary file
     * 
     * @throws IOException
     * @throws PetalsException
     */
    @Test
    public void testGetEntryAsTemp1() throws IOException {
        try {
            ZipUtil.getEntryAsTemp(new ZipFile(testResourcesDir.getAbsolutePath() + File.separator
                    + "test-ziputil.zip"), "META-INF" + File.separator + "manifest.mf");
            fail();
        } catch (PetalsException e) {
        }
    }

    /**
     * Test exception throwing
     * 
     */
    @Test
    public void testGetEntryAsTemp2() {
        try {
            ZipUtil.getEntryAsTemp((ZipFile) null, "META-INF" + File.separator + "manifest.mf");
            fail();
        } catch (NullPointerException e) {
            // NOP, expected result
        } catch (IOException e) {
            fail("PetalsException raised: " + e.getMessage());
        } catch (PetalsException e) {
            fail("PetalsException raised: " + e.getMessage());
        }
    }

    /**
     * Test exception throwing
     * 
     */
    @Test
    public void testGetEntryAsTemp3() {
        try {
            ZipUtil.getEntryAsTemp(new ZipFile(testResourcesDir.getAbsolutePath() + File.separator
                    + "test-ziputil.zip"), (String) null);
            fail();
        } catch (NullPointerException e) {
            // NOP, expected result
        } catch (PetalsException e) {
            fail("PetalsException raised: " + e.getMessage());
        } catch (IOException e) {
            fail("IOException raised: " + e.getMessage());
        }
    }

    /**
     * Test exception throwing
     * 
     */
    @Test
    public void testGetEntryAsTemp4() {
        try {
            ZipUtil.getEntryAsTemp(new ZipFile(testResourcesDir.getAbsolutePath() + File.separator
                    + "test-ziputil.zip"), "");
            fail();
        } catch (PetalsException e) {
            // expected result
        } catch (IOException e) {
            fail("IOException raised: " + e.getMessage());
        }
    }

}
